@@ -4,7 +4,7 @@
    Unix SMB/Netbios implementation.
    Version 1.9.
    Machine customisation and include handling
-   Copyright (C) Andrew Tridgell 1994-1997
+   Copyright (C) Andrew Tridgell 1994-1998
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,6 +26,11 @@
    and add a section for the new unix below.
 */
 
+#ifdef NTDOMAIN
+#error WARNING: remove this option (-DNTDOMAIN) from your [Mm]akefile.
+#error Please read NTDOMAIN.txt with _this_ distribution.
+#endif
+
 
 /* the first OS dependent section is to setup what includes will be used.
    the main OS dependent section comes later on 
@@ -193,7 +198,7 @@
 #include <pwdadj.h>
 #endif
 
-#if defined(SHADOW_PWD) && !defined(NETBSD) && !defined(FreeBSD) && !defined(CONVEX)
+#if defined(SHADOW_PWD) && !defined(NETBSD) && !defined(FreeBSD) && !defined(CONVEX) && !defined(__OpenBSD__)
 #include <shadow.h>
 #endif
 
@@ -214,9 +219,6 @@ Here come some platform specific sections
 #include <string.h>
 #include <sys/vfs.h>
 #include <netinet/in.h>
-#ifndef NO_ASMSIGNALH
-#include <asm/signal.h>
-#endif
 #ifdef GLIBC2
 #define _LINUX_C_LIB_VERSION_MAJOR     6
 #include <termios.h>
@@ -225,13 +227,18 @@ Here come some platform specific sections
 #include <netinet/tcp.h>
 #include <netinet/ip.h>
 #endif
+#ifndef QSORT_CAST
+#define QSORT_CAST (int (*)(const void *, const void *))
+#endif /* QSORT_CAST */
 #define SIGNAL_CAST (__sighandler_t)
 #define USE_GETCWD
 #define USE_SETSID
 #define HAVE_BZERO
 #define HAVE_MEMMOVE
+#define HAVE_VSNPRINTF
 #define USE_SIGPROCMASK
 #define USE_WAITPID
+#define USE_SYSV_IPC
 #if 0
 /* SETFS disabled until we can check on some bug reports */
 #if _LINUX_C_LIB_VERSION_MAJOR >= 5
@@ -254,6 +261,7 @@ Here come some platform specific sections
 #include <sys/acct.h>
 #include <sys/vfs.h>
 #include <string.h>
+#include <strings.h>
 #include <errno.h>
 #include <sys/wait.h>
 #include <signal.h>
@@ -279,8 +287,10 @@ typedef unsigned short mode_t;
 #ifndef USE_WAITPID
 #define USE_WAITPID
 #endif
+#define USE_SYSV_IPC
 /* SunOS doesn't have POSIX atexit */
 #define atexit on_exit
+#define NOSTRCASECMP
 #endif
 
 
@@ -300,9 +310,9 @@ typedef unsigned short mode_t;
 #include <rpcsvc/ypclnt.h>
 #include <termios.h>
 #include <sys/stropts.h>
-#ifndef USE_LIBDES
+#if !defined(USE_LIBDES) && !defined(KRB4_AUTH)
 #include <crypt.h>
-#endif /* USE_LIBDES */
+#endif /* !USE_LIBDES && !KRB4_AUTH */
 extern int gettimeofday (struct timeval *, void *);
 extern int gethostname (char *name, int namelen);
 extern int innetgr (const char *, const char *, const char *, const char *);
@@ -316,6 +326,8 @@ extern int innetgr (const char *, const char *, const char *, const char *);
 #define USE_STATVFS
 #define USE_GETCWD
 #define USE_SETSID
+#define USE_SYSV_IPC
+#define NO_SEMUN
 #ifndef REPLACE_GETPASS
 #define REPLACE_GETPASS
 #endif /* REPLACE_GETPASS */
@@ -340,7 +352,7 @@ char *getwd(char *);
 #define USE_WAITPID
 #endif
 
-#ifdef SGI
+#ifdef SGI4
 #include <netinet/tcp.h>
 #include <sys/statfs.h>
 #include <string.h>
@@ -353,9 +365,10 @@ char *getwd(char *);
 #define USE_WAITPID
 #define USE_DIRECT
 #define USE_SETSID
+#define USE_SYSV_IPC
 #endif
 
-#ifdef SGI5
+#if defined(SGI5) || defined(SGI6)
 #include <arpa/inet.h>
 #include <netinet/tcp.h>
 #include <netinet/in_systm.h>
@@ -373,6 +386,10 @@ char *getwd(char *);
 #define USE_STATVFS
 #define USE_WAITPID
 #define USE_SETSID
+#define USE_SYSV_IPC
+#ifndef QSORT_CAST
+#define QSORT_CAST (int (*)(const void *, const void *))
+#endif /* QSORT_CAST */
 #endif
 
 
@@ -429,6 +446,8 @@ extern struct passwd *getpwnam();
 #define USE_STATVFS
 #define USE_GETCWD
 #define USE_SETSID
+#define USE_SYSV_IPC
+#define NO_SEMUN
 #endif
 
 
@@ -454,6 +473,8 @@ char *mktemp(char *); /* No standard include */
 #define PASSWORD_LENGTH 16
 #define NEED_AUTH_PARAMETERS
 #endif  /* OSF1_ENH_SEC */
+#define USE_SYSV_IPC
+#define NO_SEMUN
 #endif
 
 
@@ -485,8 +506,15 @@ char *mktemp(char *); /* No standard include */
 #endif
 
 
-#ifdef NETBSD
+#ifdef NETBSD 
+#ifdef NetBSD1_3
+#include <string.h>
+#ifdef ALLOW_CHANGE_PASSWORD
+#include <termios.h>
+#endif /* ALLOW_CHANGE_PASSWORD */
+#else /* NetBSD1_3 */
 #include <strings.h>
+#endif /* NetBSD1_3 */
 #include <netinet/tcp.h>
 /* you may not need this */
 #define NO_GETSPNAM
@@ -503,21 +531,51 @@ char *mktemp(char *); /* No standard include */
 #include <netinet/tcp.h>
 #include <netinet/in_systm.h>
 #include <netinet/ip.h>
-#define SIGNAL_CAST (void (*)())
+#include <termios.h>
+#if __FreeBSD__ >= 3
+#include <dirent.h>
+#else
+#define USE_DIRECT
+#endif
+#define SIGNAL_CAST (void (*)(int))
 #define USE_SETVBUF
 #define USE_SETSID
 #define USE_GETCWD
 #define USE_WAITPID
-#define USE_DIRECT
 #define HAVE_MEMMOVE
 #define HAVE_BZERO
 #define HAVE_GETTIMEOFDAY
 #define HAVE_PATHCONF
 #define HAVE_GETGRNAM 1
+#ifndef QSORT_CAST
+#define QSORT_CAST (int (*)(const void *, const void *))
+#endif /* QSORT_CAST */
+#if !defined(O_SYNC)
+#if defined(O_FSYNC)
+#define O_SYNC O_FSYNC
+#else /* defined(O_FSYNC) */
+#define O_SYNC 0
+#endif /* defined(O_FSYNC) */
+#endif /* !defined(O_SYNC) */
+#define HAVE_VSNPRINTF
+#endif /* FreeBSD */
+
+#ifdef __OpenBSD__
+#include <strings.h>
+#include <netinet/tcp.h>
+#define NO_GETSPNAM
+#define SIGNAL_CAST (void (*)())
+#define USE_DIRECT
+#define REPLACE_INNETGR
+#define HAVE_BZERO
+#define HAVE_PATHCONF
+#define HAVE_GETGRNAM 1
+#define HAVE_GETTIMEOFDAY
+#define HAVE_MEMMOVE
+#define USE_GETCWD
+#define USE_SETSID
 #endif 
 
-
-
 #ifdef AIX
 #include <strings.h>
 #include <sys/dir.h>
@@ -527,6 +585,10 @@ char *mktemp(char *); /* No standard include */
 #include <sys/vfs.h>
 #include <sys/id.h>
 #include <sys/priv.h>
+/* According to AIX 4.1 man pages, inet_ntoa needs the following headers */
+#include <sys/socket.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
 #include <netinet/tcp.h>
 #include <locale.h>
 #define SYSV
@@ -547,6 +609,8 @@ char *mktemp(char *); /* No standard include */
 #include <sys/types.h>
 #include <sys/termios.h>
 #include <netinet/tcp.h>
+#include <arpa/inet.h>                /* needed for inet_ntoa proto */
+#include <stropts.h>
 #ifdef HPUX_10_TRUSTED
 #include <hpsecurity.h>
 #include <prot.h>
@@ -562,8 +626,13 @@ char *mktemp(char *); /* No standard include */
 #define USE_GETCWD
 #define USE_SETSID
 #define USE_SETRES
+#define USE_SYSV_IPC
+#define NO_SEMUN
 #define DEFAULT_PRINTING PRINT_HPUX
-#define SIGCLD_IGNORE
+/* Ken Weiss <krweiss@ucdavis.edu> tells us that SIGCLD_IGNORE is
+   not good for HPUX */
+/* #define SIGCLD_IGNORE */
+#define USE_SIGPROCMASK /* Needed to stop zombie processes on HPUX 9.x and 10.x.*/
 #endif /* HPUX */
 
 
@@ -673,6 +742,7 @@ char *mktemp(char *); /* No standard include */
 #include <sys/netinet/ip.h>
 #include <dirent.h>
 #include <string.h>
+#include <termios.h>
 #include <fcntl.h>
 #include <sys/statfs.h>
 #include <sys/stropts.h>
@@ -1061,6 +1131,40 @@ struct spwd { /* fake shadow password structure */
 #endif
 #endif
 
+/* This defines the name of the printcap file. It is MOST UNLIKELY that
+   this will change BUT! Specifying a file with the format of a printcap
+   file but containing only a subset of the printers actually in your real 
+   printcap file is a quick-n-dirty way to allow dynamic access to a subset
+   of available printers.
+*/
+#ifndef PRINTCAP_NAME
+#ifdef AIX
+#define PRINTCAP_NAME "/etc/qconfig"
+#elif defined(SYSV)
+#define PRINTCAP_NAME "lpstat"
+#else
+#define PRINTCAP_NAME "/etc/printcap"
+#endif
+#endif
+
+
+#ifdef USE_SYSV_IPC
+#include <sys/ipc.h>
+#include <sys/sem.h>
+#include <sys/shm.h>
+#ifdef NO_SEMUN
+union semun {
+	int val;
+	struct semid_ds *buf;
+	unsigned short *array;
+};
+#endif
+#if defined(HPUX) && defined(HPUX10)
+#ifdef SEMMSL
+#undef SEMMSL
+#endif /* SEMMSL */
+#endif /* HPUX && HPUX10 */
+#endif
 
 #ifdef AFS_AUTH
 #include <afs/stds.h>
@@ -1076,6 +1180,10 @@ struct spwd { /* fake shadow password structure */
 #include <krb5.h>
 #endif
 
+#ifdef KRB4_AUTH
+#include <krb.h>
+#endif
+
 #ifdef NO_UTIMBUF
 struct utimbuf {
   time_t actime;
@@ -1101,26 +1209,36 @@ extern char *sys_errlist[];
 #include "version.h"
 #include "smb.h"
 #include "nameserv.h"
-#include "proto.h"
+#include "ubiqx/ubi_dLinkList.h"
+
 #include "byteorder.h"
 
 #include "kanji.h"
 #include "charset.h"
 
+#ifndef MAXCODEPAGELINES
+#define MAXCODEPAGELINES 256
+#endif
+
+/***** automatically generated prototypes *****/
+#include "proto.h"
+
+
+
 #ifndef S_IFREG
 #define S_IFREG 0100000
 #endif
 
 #ifndef S_ISREG
-#define S_ISREG(x) ((S_IFREG & x)!=0)
+#define S_ISREG(x) ((S_IFREG & (x))!=0)
 #endif
 
 #ifndef S_ISDIR
-#define S_ISDIR(x) ((S_IFDIR & x)!=0)
+#define S_ISDIR(x) ((S_IFDIR & (x))!=0)
 #endif
 
 #if !defined(S_ISLNK) && defined(S_IFLNK)
-#define S_ISLNK(x) ((S_IFLNK & x)!=0)
+#define S_ISLNK(x) ((S_IFLNK & (x))!=0)
 #endif
 
 #ifdef UFC_CRYPT
@@ -1191,12 +1309,16 @@ it works and getting lots of bug reports */
 #endif
 
 #ifndef QSORT_CAST
-#define QSORT_CAST (int (*)())
+#define QSORT_CAST (int (*)(void *, void *))
 #endif
 
+#ifndef INADDR_LOOPBACK
+#define INADDR_LOOPBACK 0x7f000001
+#endif /* INADDR_LOOPBACK */
+
 /* this is a rough check to see if this machine has a lstat() call.
    it is not guaranteed to work */
-#if !(defined(S_ISLNK) || defined(S_IFLNK))
+#if !defined(S_ISLNK)
 #define lstat stat
 #endif
 
@@ -1230,55 +1352,28 @@ extern int errno;
 #define strncasecmp(s1,s2,n) StrnCaseCmp(s1,s2,n)
 #endif
 
-#ifndef strcpy
-#define strcpy(dest,src) StrCpy(dest,src)
-#endif
+#ifdef strcpy
+#undef strcpy
+#endif /* strcpy */
+#define strcpy(dest,src) __ERROR__XX__NEVER_USE_STRCPY___;
+   
+#ifdef strcat
+#undef strcat
+#endif /* strcat */
+#define strcat(dest,src) __ERROR__XX__NEVER_USE_STRCAT___;
+   
+#ifdef sprintf
+#undef sprintf
+#endif /* sprintf */
+#define sprintf __ERROR__XX__NEVER_USE_SPRINTF__;
 
+#define pstrcpy(d,s) safe_strcpy((d),(s),sizeof(pstring)-1)
+#define pstrcat(d,s) safe_strcat((d),(s),sizeof(pstring)-1)
+#define fstrcpy(d,s) safe_strcpy((d),(s),sizeof(fstring)-1)
+#define fstrcat(d,s) safe_strcat((d),(s),sizeof(fstring)-1)
 
-/* possibly wrap the malloc calls */
-#if WRAP_MALLOC
-
-/* undo the old malloc def if necessary */
-#ifdef malloc
-#define xx_old_malloc malloc
-#undef malloc
-#endif
-
-#define malloc(size) malloc_wrapped(size,__FILE__,__LINE__)
-
-/* undo the old realloc def if necessary */
-#ifdef realloc
-#define xx_old_realloc realloc
-#undef realloc
-#endif
-
-#define realloc(ptr,size) realloc_wrapped(ptr,size,__FILE__,__LINE__)
-
-/* undo the old free def if necessary */
-#ifdef free
-#define xx_old_free free
-#undef free
-#endif
-
-#define free(ptr) free_wrapped(ptr,__FILE__,__LINE__)
-
-/* and the malloc prototypes */
-void *malloc_wrapped(int,char *,int);
-void *realloc_wrapped(void *,int,char *,int);
-void free_wrapped(void *,char *,int);
-
-#endif
-
-
-#if WRAP_MEMCPY
-/* undo the old memcpy def if necessary */
-#ifdef memcpy
-#define xx_old_memcpy memcpy
-#undef memcpy
-#endif
-
-#define memcpy(d,s,l) memcpy_wrapped(d,s,l,__FILE__,__LINE__)
-void *memcpy_wrapped(void *d,void *s,int l,char *fname,int line);
+#if MEM_MAN
+#include "mem_man/mem_man.h"
 #endif
 
 #endif
