@@ -2,7 +2,7 @@
    Unix SMB/Netbios implementation.
    Version 1.9.
    multiple interface handling
-   Copyright (C) Andrew Tridgell 1992-1997
+   Copyright (C) Andrew Tridgell 1992-1998
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
 extern int DEBUGLEVEL;
 
 struct in_addr ipzero;
-struct in_addr wins_ip;
+struct in_addr allones_ip;
 struct in_addr loopback_ip;
 static struct in_addr default_ip;
 static struct in_addr default_bcast;
@@ -33,34 +33,40 @@ static BOOL got_ip=False;
 static BOOL got_bcast=False;
 static BOOL got_nmask=False;
 
-struct interface *local_interfaces  = NULL;
+static struct interface *local_interfaces  = NULL;
 
 struct interface *last_iface;
 
+#define ALLONES  ((uint32)0xFFFFFFFF)
+#define MKBCADDR(_IP, _NM) ((_IP & _NM) | (_NM ^ ALLONES))
 /****************************************************************************
 calculate the default netmask for an address
 ****************************************************************************/
 static void default_netmask(struct in_addr *inm, struct in_addr *iad)
 {
-  uint32 ad = ntohl(iad->s_addr);
-  uint32 nm;
-  /*
-  ** Guess a netmask based on the class of the IP address given.
-  */
-  if ( (ad & 0x80000000) == 0 ) {
-    /* class A address */
-    nm = 0xFF000000;
-  } else if ( (ad & 0xC0000000) == 0x80000000 ) {
-    /* class B address */
-    nm = 0xFFFF0000;
-  } else if ( (ad & 0xE0000000) == 0xC0000000 ) {
-    /* class C address */
-    nm = 0xFFFFFF00;
-  }  else {
-    /* class D or E; netmask doesn't make much sense - guess 4 bits */
-    nm =  0xFFFFFFF0;
-  }
-  inm->s_addr = htonl(nm);
+	/*
+	** Guess a netmask based on the class of the IP address given.
+	*/
+	switch((ntohl(iad->s_addr) & 0xE0000000)) {
+        case 0x00000000:     /* Class A addr */
+        case 0x20000000:
+        case 0x40000000:
+        case 0x60000000:
+		inm->s_addr = htonl(0xFF000000);
+		break;
+		
+	case 0x80000000:	/* Class B addr */
+        case 0xA0000000:
+		inm->s_addr = htonl(0xFFFF0000);
+		break;
+		
+	case 0xC0000000:	/* Class C addr */
+		inm->s_addr = htonl(0xFFFFFF00);
+		break;
+		
+	default:		/* ??? */
+		inm->s_addr = htonl(0xFFFFFFF0);
+        }
 }
 
 
@@ -155,7 +161,7 @@ static void get_broadcast(struct in_addr *if_ipaddr,
       }
     }
   }
-#elif defined(__FreeBSD__) || defined(NETBSD) || defined(AMIGA) || defined(_AIX41)
+#elif defined(__FreeBSD__) || defined(NETBSD) || defined(AMIGA) || defined(_AIX41) || defined(__OpenBSD__)
   ifc.ifc_len = sizeof(buff);
   ifc.ifc_buf = buff;
   if (ioctl(sock, SIOCGIFCONF, &ifc) < 0) {
@@ -228,8 +234,30 @@ static void get_broadcast(struct in_addr *if_ipaddr,
 
   /* sanity check on the netmask */
   {
-    uint32 nm = ntohl(if_nmask->s_addr);
-    if ((nm >> 24) != 0xFF) {
+    uint32 nm;
+    short onbc;
+    short offbc;
+
+    nm = ntohl(if_nmask->s_addr);
+    onbc = 0;
+    offbc = 0;
+    while( (onbc + offbc) < 32 )
+         {
+           if( nm & 0x80000000 )
+             {
+               onbc++;
+               if( offbc ) /* already found an off bit, so mask is wrong */
+                 {
+                   onbc = 34;
+                 }
+             }
+           else
+             {
+               offbc++;
+             }
+           nm <<= 1;
+         }
+    if ((onbc < 8)||(onbc == 34)) {
       DEBUG(0,("Impossible netmask %s - using defaults\n",inet_ntoa(*if_nmask)));
       default_netmask(if_nmask, if_ipaddr);      
     }
@@ -239,10 +267,7 @@ static void get_broadcast(struct in_addr *if_ipaddr,
      all MS operating systems do, we have to comply even if the unix
      box is setup differently */
   {
-    uint32 ad = ntohl(if_ipaddr->s_addr);
-    uint32 nm = ntohl(if_nmask->s_addr);
-    uint32 bc = (ad & nm) | (0xffffffff & ~nm);
-    if_bcast->s_addr = htonl(bc);
+    if_bcast->s_addr = MKBCADDR(if_ipaddr->s_addr, if_nmask->s_addr);
   }
   
   DEBUG(4,("Derived broadcast address %s\n", inet_ntoa(*if_bcast)));
@@ -256,19 +281,20 @@ load a list of network interfaces
 static void interpret_interfaces(char *s, struct interface **interfaces,
 		char *description)
 {
-  char *ptr = s;
+  char *ptr;
   fstring token;
   struct interface *iface;
   struct in_addr ip;
 
+  ptr = s;
   ipzero = *interpret_addr2("0.0.0.0");
-  wins_ip = *interpret_addr2("255.255.255.255");
+  allones_ip = *interpret_addr2("255.255.255.255");
   loopback_ip = *interpret_addr2("127.0.0.1");
 
   while (next_token(&ptr,token,NULL)) {
     /* parse it into an IP address/netmasklength pair */
     char *p = strchr(token,'/');
-    if (p) *p = 0;
+    if (p) *p++ = 0;
 
     ip = *interpret_addr2(token);
 
@@ -286,14 +312,14 @@ static void interpret_interfaces(char *s, struct interface **interfaces,
     iface->ip = ip;
 
     if (p) {
-      if (strlen(p+1)>2)
-	iface->nmask = *interpret_addr2(p+1);
+      if (strlen(p) > 2)
+       iface->nmask = *interpret_addr2(p);
       else
-	iface->nmask.s_addr = htonl(~((1<<(32-atoi(p+1)))-1));
+       iface->nmask.s_addr = htonl(((ALLONES >> atoi(p)) ^ ALLONES));
     } else {
       default_netmask(&iface->nmask,&iface->ip);
     }
-    iface->bcast.s_addr = iface->ip.s_addr | ~iface->nmask.s_addr;
+    iface->bcast.s_addr = MKBCADDR(iface->ip.s_addr, iface->nmask.s_addr);
     iface->next = NULL;
 
     if (!(*interfaces)) {
@@ -302,9 +328,9 @@ static void interpret_interfaces(char *s, struct interface **interfaces,
       last_iface->next = iface;
     }
     last_iface = iface;
-    DEBUG(1,("Added %s ip=%s ",description,inet_ntoa(iface->ip)));
-    DEBUG(1,("bcast=%s ",inet_ntoa(iface->bcast)));
-    DEBUG(1,("nmask=%s\n",inet_ntoa(iface->nmask)));	     
+    DEBUG(2,("Added %s ip=%s ",description,inet_ntoa(iface->ip)));
+    DEBUG(2,("bcast=%s ",inet_ntoa(iface->bcast)));
+    DEBUG(2,("nmask=%s\n",inet_ntoa(iface->nmask)));	     
   }
 
   if (*interfaces) return;
@@ -329,10 +355,10 @@ static void interpret_interfaces(char *s, struct interface **interfaces,
 
   if (got_nmask) {
     iface->nmask = default_nmask;
-    iface->bcast.s_addr = iface->ip.s_addr | ~iface->nmask.s_addr;
+    iface->bcast.s_addr = MKBCADDR(iface->ip.s_addr, iface->nmask.s_addr);
   }
 
-  if (iface->bcast.s_addr != (iface->ip.s_addr | ~iface->nmask.s_addr)) {
+  if (iface->bcast.s_addr != MKBCADDR(iface->ip.s_addr, iface->nmask.s_addr)) {
     DEBUG(2,("Warning: inconsistant interface %s\n",inet_ntoa(iface->ip)));
   }
 
@@ -399,6 +425,18 @@ BOOL ismybcast(struct in_addr bcast)
   return False;
 }
 
+/****************************************************************************
+  check if a packet is from a local (known) net
+  **************************************************************************/
+BOOL is_local_net(struct in_addr from)
+{
+  struct interface *i;
+  for (i=local_interfaces;i;i=i->next)
+    if((from.s_addr & i->nmask.s_addr) == (i->ip.s_addr & i->nmask.s_addr))
+      return True;
+  return False;
+}
+
 /****************************************************************************
   how many interfaces do we have
   **************************************************************************/
@@ -412,6 +450,33 @@ int iface_count(void)
   return ret;
 }
 
+/****************************************************************************
+ True if we have two or more interfaces.
+  **************************************************************************/
+BOOL we_are_multihomed(void)
+{
+  static int multi = -1;
+
+  if(multi == -1)
+    multi = (iface_count() > 1 ? True : False);
+
+  return multi;
+}
+
+/****************************************************************************
+  return the Nth interface
+  **************************************************************************/
+struct interface *get_interface(int n)
+{ 
+  struct interface *i;
+  
+  for (i=local_interfaces;i && n;i=i->next)
+    n--;
+
+  if (i) return i;
+  return NULL;
+}
+
 /****************************************************************************
   return IP of the Nth interface
   **************************************************************************/
@@ -426,6 +491,9 @@ struct in_addr *iface_n_ip(int n)
   return NULL;
 }
 
+/****************************************************************************
+Try and find an interface that matches an ip. If we cannot, return NULL
+  **************************************************************************/
 static struct interface *iface_find(struct in_addr ip)
 {
   struct interface *i;
@@ -434,25 +502,30 @@ static struct interface *iface_find(struct in_addr ip)
   for (i=local_interfaces;i;i=i->next)
     if (same_net(i->ip,ip,i->nmask)) return i;
 
-  return local_interfaces;
+  return NULL;
 }
 
 /* these 3 functions return the ip/bcast/nmask for the interface
-   most appropriate for the given ip address */
+   most appropriate for the given ip address. If they can't find
+   an appropriate interface they return the requested field of the
+   first known interface. */
 
 struct in_addr *iface_bcast(struct in_addr ip)
 {
-  return(&iface_find(ip)->bcast);
+  struct interface *i = iface_find(ip);
+  return(i ? &i->bcast : &local_interfaces->bcast);
 }
 
 struct in_addr *iface_nmask(struct in_addr ip)
 {
-  return(&iface_find(ip)->nmask);
+  struct interface *i = iface_find(ip);
+  return(i ? &i->nmask : &local_interfaces->nmask);
 }
 
 struct in_addr *iface_ip(struct in_addr ip)
 {
-  return(&iface_find(ip)->ip);
+  struct interface *i = iface_find(ip);
+  return(i ? &i->ip : &local_interfaces->ip);
 }
 
 
