@@ -52,8 +52,6 @@ extern fstring myworkgroup;
 #define ERROR_INVALID_LEVEL 124
 #define ERROR_MORE_DATA 234
 
-#define REALLOC(ptr,size) Realloc(ptr,MAX((size),4*1024))
-
 #define ACCESS_READ 0x01
 #define ACCESS_WRITE 0x02
 #define ACCESS_CREATE 0x04
@@ -64,6 +62,8 @@ extern fstring myworkgroup;
 #define QNLEN 12		/* queue name maximum length */
 
 extern int Client;
+extern int oplock_sock;
+extern int smb_read_error;
 
 static BOOL api_Unsupported(int cnum,uint16 vuid, char *param,char *data,
 			    int mdrcnt,int mprcnt,
@@ -146,7 +146,12 @@ static void send_trans_reply(char *outbuf,char *data,char *param,uint16 *setup,
   this_lparam = MIN(lparam,max_send - (500+lsetup*SIZEOFWORD)); /* hack */
   this_ldata = MIN(ldata,max_send - (500+lsetup*SIZEOFWORD+this_lparam));
 
+#ifdef CONFUSE_NETMONITOR_MSRPC_DECODING
+  /* if you don't want Net Monitor to decode your packets, do this!!! */
+  align = ((this_lparam+1)%4);
+#else
   align = (this_lparam%4);
+#endif
 
   set_message(outbuf,10+lsetup,align+this_ldata+this_lparam,True);
   if (this_lparam)
@@ -278,8 +283,8 @@ static BOOL init_package(struct pack_desc* p, int count, int subcount)
     i = n = 0;
     p->errcode = NERR_BufTooSmall;
   }
-
-  p->errcode = NERR_Success;
+  else
+    p->errcode = NERR_Success;
   p->buflen = i;
   n -= i;
   p->stringbuf = p->base + i;
@@ -440,7 +445,7 @@ static void PackDriverData(struct pack_desc* desc)
 }
 
 static int check_printq_info(struct pack_desc* desc,
- 			     int uLevel, char *id1, const char* id2)
+ 			     int uLevel, char *id1, char *id2)
 {
   desc->subformat = NULL;
   switch( uLevel ) {
@@ -464,6 +469,10 @@ static int check_printq_info(struct pack_desc* desc,
   case 5:
     desc->format = "z";
     break;
+  case 52:
+    desc->format = "WzzzzzzzzN";
+    desc->subformat = "z";
+    break;
   default: return False;
   }
   if (strcmp(desc->format,id1) != 0) return False;
@@ -480,7 +489,7 @@ static void fill_printjob_info(int cnum, int snum, int uLevel,
   /* the client expects localtime */
   t -= TimeDiff(t);
 
-  PACKI(desc,"W",((snum%0xFF)<<8) | (queue->job%0xFF)); /* uJobId */
+  PACKI(desc,"W",printjob_encode(snum, queue->job)); /* uJobId */
   if (uLevel == 1) {
     PACKS(desc,"B21",queue->user); /* szUserName */
     PACKS(desc,"B","");		/* pad */
@@ -523,11 +532,18 @@ static void fill_printq_info(int cnum, int snum, int uLevel,
  			     int count, print_queue_struct* queue,
  			     print_status_struct* status)
 {
-  if (uLevel < 3) {
-    PACKS(desc,"B13",SERVICE(snum));
-  } else {
-    PACKS(desc,"z",Expand(cnum,snum,SERVICE(snum)));
+  switch (uLevel) {
+    case 1:
+    case 2:
+      PACKS(desc,"B13",SERVICE(snum));
+      break;
+    case 3:
+    case 4:
+    case 5:
+      PACKS(desc,"z",Expand(cnum,snum,SERVICE(snum)));
+      break;
   }
+
   if (uLevel == 1 || uLevel == 2) {
     PACKS(desc,"B","");		/* alignment */
     PACKI(desc,"W",5);		/* priority */
@@ -575,8 +591,150 @@ static void fill_printq_info(int cnum, int snum, int uLevel,
     for (i=0;i<count;i++)
       fill_printjob_info(cnum,snum,uLevel == 2 ? 1 : 2,desc,&queue[i],i);
   }
- 
-  DEBUG(3,("fill_printq_info on <%s> gave %d entries\n",SERVICE(snum),count));
+
+  if (uLevel==52) {
+    int i,ok=0;
+    pstring tok,driver,datafile,langmon,helpfile,datatype;
+    char *p,*q;
+    FILE *f;
+    pstring fname;
+
+    strcpy(fname,lp_driverfile());
+    f=fopen(fname,"r");
+    if (!f) {
+      DEBUG(3,("fill_printq_info: Can't open %s - %s\n",fname,strerror(errno)));
+      desc->errcode=NERR_notsupported;
+      return;
+    }
+
+    p=(char *)malloc(8192*sizeof(char));
+    bzero(p, 8192*sizeof(char));
+    q=p;
+
+    /* lookup the long printer driver name in the file description */
+    while (f && !feof(f) && !ok)
+    {
+      p = q;			/* reset string pointer */
+      fgets(p,8191,f);
+      p[strlen(p)-1]='\0';
+      if (next_token(&p,tok,":") &&
+        (!strncmp(tok,lp_printerdriver(snum),strlen(lp_printerdriver(snum)))))
+	ok=1;
+    }
+    fclose(f);
+
+    /* driver file name */
+    if (ok && !next_token(&p,driver,":")) ok = 0;
+    /* data file name */
+    if (ok && !next_token(&p,datafile,":")) ok = 0;
+      /*
+       * for the next tokens - which may be empty - I have to check for empty
+       * tokens first because the next_token function will skip all empty
+       * token fields 
+       */
+    if (ok) {
+      /* help file */
+      if (*p == ':') {
+	  *helpfile = '\0';
+	  p++;
+      } else if (!next_token(&p,helpfile,":")) ok = 0;
+    }
+
+    if (ok) {
+      /* language monitor */
+      if (*p == ':') {
+	  *langmon = '\0';
+	  p++;
+      } else if (!next_token(&p,langmon,":")) ok = 0;
+    }
+
+    /* default data type */
+    if (ok && !next_token(&p,datatype,":")) ok = 0;
+
+    if (ok) {
+      PACKI(desc,"W",0x0400);                    /* don't know */
+      PACKS(desc,"z",lp_printerdriver(snum));    /* long printer name */
+      PACKS(desc,"z",driver);                    /* Driverfile Name */
+      PACKS(desc,"z",datafile);                  /* Datafile name */
+      PACKS(desc,"z",langmon);			 /* language monitor */
+      PACKS(desc,"z",lp_driverlocation(snum));   /* share to retrieve files */
+      PACKS(desc,"z",datatype);			 /* default data type */
+      PACKS(desc,"z",helpfile);                  /* helpfile name */
+      PACKS(desc,"z",driver);                    /* driver name */
+      DEBUG(3,("Driver:%s:\n",driver));
+      DEBUG(3,("Data File:%s:\n",datafile));
+      DEBUG(3,("Language Monitor:%s:\n",langmon));
+      DEBUG(3,("Data Type:%s:\n",datatype));
+      DEBUG(3,("Help File:%s:\n",helpfile));
+      PACKI(desc,"N",count);                     /* number of files to copy */
+      for (i=0;i<count;i++)
+      {
+	/* no need to check return value here - it was already tested in
+	 * get_printerdrivernumber
+	 */
+        next_token(&p,tok,",");
+        PACKS(desc,"z",tok);                        /* driver files to copy */
+        DEBUG(3,("file:%s:\n",tok));
+      }
+
+      DEBUG(3,("fill_printq_info on <%s> gave %d entries\n",
+	    SERVICE(snum),count));
+    } else {
+      DEBUG(3,("fill_printq_info: Can't supply driver files\n"));
+      desc->errcode=NERR_notsupported;
+    }
+    free(q);
+  }
+}
+
+/* This function returns the number of files for a given driver */
+int get_printerdrivernumber(int snum)
+{
+  int i=0,ok=0;
+  pstring tok;
+  char *p,*q;
+  FILE *f;
+  pstring fname;
+
+  strcpy(fname,lp_driverfile());
+
+  DEBUG(4,("In get_printerdrivernumber: %s\n",fname));
+  f=fopen(fname,"r");
+  if (!f) {
+    DEBUG(3,("get_printerdrivernumber: Can't open %s - %s\n",fname,strerror(errno)));
+    return(0);
+  }
+
+  p=(char *)malloc(8192*sizeof(char));
+  q=p; /* need it to free memory because p change ! */
+
+  /* lookup the long printer driver name in the file description */
+  while (!feof(f) && !ok)
+  {
+    p = q;			/* reset string pointer */
+    fgets(p,8191,f);
+    if (next_token(&p,tok,":") &&
+      (!strncmp(tok,lp_printerdriver(snum),strlen(lp_printerdriver(snum))))) 
+	ok=1;
+  }
+  fclose(f);
+
+  if (ok) {
+    /* skip 5 fields */
+    i = 5;
+    while (*p && i) {
+      if (*p++ == ':') i--;
+    }
+    if (!*p || i)
+      return(0);
+
+    /* count the number of files */
+    while (next_token(&p,tok,","))
+       i++;
+  }
+  free(q);
+
+  return(i);
 }
 
 static BOOL api_DosPrintQGetInfo(int cnum,uint16 vuid, char *param,char *data,
@@ -624,7 +782,14 @@ static BOOL api_DosPrintQGetInfo(int cnum,uint16 vuid, char *param,char *data,
   
   if (snum < 0 || !VALID_SNUM(snum)) return(False);
 
-  count = get_printqueue(snum,cnum,&queue,&status);
+  if (uLevel==52)
+  {
+    count = get_printerdrivernumber(snum);
+    DEBUG(3,("api_DosPrintQGetInfo: Driver files count: %d\n",count));
+  }
+  else
+    count = get_printqueue(snum,cnum,&queue,&status);
+
   if (mdrcnt > 0) *rdata = REALLOC(*rdata,mdrcnt);
   desc.base = *rdata;
   desc.buflen = mdrcnt;
@@ -774,7 +939,7 @@ static int get_server_info(uint32 servertype,
   pstring line;
   BOOL local_list_only;
 
-  strcpy(fname,lp_lockdir());
+  pstrcpy(fname,lp_lockdir());
   trim_string(fname,NULL,"/");
   strcat(fname,"/");
   strcat(fname,SERVER_LIST);
@@ -1074,7 +1239,7 @@ static BOOL api_RNetServerEnum(int cnum, uint16 vuid, char *param, char *data,
   
   *rparam_len = 8;
   *rparam = REALLOC(*rparam,*rparam_len);
-  SSVAL(*rparam,0,NERR_Success);
+  SSVAL(*rparam,0,(missed == 0 ? NERR_Success : ERROR_MORE_DATA));
   SSVAL(*rparam,2,0);
   SSVAL(*rparam,4,counted);
   SSVAL(*rparam,6,counted+missed);
@@ -1363,7 +1528,7 @@ static BOOL api_SetUserPassword(int cnum,uint16 vuid, char *param,char *data,
   fstring user;
   fstring pass1,pass2;
 
-  strcpy(user,p);
+  fstrcpy(user,p);
 
   p = skip_string(p,1);
 
@@ -1405,11 +1570,10 @@ static BOOL api_RDosPrintJobDel(int cnum,uint16 vuid, char *param,char *data,
   char *str1 = param+2;
   char *str2 = skip_string(str1,1);
   char *p = skip_string(str2,1);
-  int jobid = (SVAL(p,0)&0xFF); /* the snum and jobid are encoded
-				   by the print queue api */
-  int snum = (SVAL(p,0)>>8);  
+  int jobid, snum;
   int i, count;
 
+  printjob_decode(SVAL(p,0), &snum, &jobid);
 
   /* check it's a supported varient */
   if (!(strcsequal(str1,"W") && strcsequal(str2,"")))
@@ -1429,7 +1593,7 @@ static BOOL api_RDosPrintJobDel(int cnum,uint16 vuid, char *param,char *data,
       count = get_printqueue(snum,cnum,&queue,NULL);
   
       for (i=0;i<count;i++)
-  	if ((queue[i].job%0xFF) == jobid)
+  	if ((queue[i].job&0xFF) == jobid)
   	  {
  	    switch (function) {
 	    case 81:		/* delete */ 
@@ -1538,13 +1702,13 @@ static BOOL api_PrintJobInfo(int cnum,uint16 vuid,char *param,char *data,
   char *str1 = param+2;
   char *str2 = skip_string(str1,1);
   char *p = skip_string(str2,1);
-  int jobid = (SVAL(p,0)&0xFF); /* the snum and jobid are encoded
- 				   by the print queue api */
-  int snum = (SVAL(p,0)>>8);
+  int jobid, snum;
   int uLevel = SVAL(p,2);
   int function = SVAL(p,4);	/* what is this ?? */
   int i;
   char *s = data;
+
+  printjob_decode(SVAL(p,0), &snum, &jobid);
    
   *rparam_len = 4;
   *rparam = REALLOC(*rparam,*rparam_len);
@@ -1565,7 +1729,7 @@ static BOOL api_PrintJobInfo(int cnum,uint16 vuid,char *param,char *data,
 	lpq_reset(snum);
 	count = get_printqueue(snum,cnum,&queue,NULL);
 	for (i=0;i<count;i++)	/* find job */
-	  if ((queue[i].job%0xFF) == jobid) break;
+	  if ((queue[i].job&0xFF) == jobid) break;
  	    
 	if (i==count) {
 	  desc.errcode=NERR_JobNotFound;
@@ -1603,21 +1767,26 @@ static BOOL api_PrintJobInfo(int cnum,uint16 vuid,char *param,char *data,
 	
 	DEBUG(3,("Setting print name to %s\n",name));
 	
+        become_root(True);
+
 	for (i=0;i<MAX_OPEN_FILES;i++)
 	  if (Files[i].open && Files[i].print_file)
 	    {
 	      pstring wd;
+          int fcnum = Files[i].cnum;
 	      GetWd(wd);
 	      unbecome_user();
 	      
-	      if (!become_user(Files[i].cnum,vuid) || 
-		  !become_service(Files[i].cnum,True))
+	      if (!become_user(&Connections[fcnum], fcnum,vuid) || 
+		  !become_service(fcnum,True))
 		break;
 	      
 	      if (sys_rename(Files[i].name,name) == 0)
 		string_set(&Files[i].name,name);
 	      break;
 	    }
+
+         unbecome_root(True);
       }
     desc.errcode=NERR_Success;
   
@@ -1699,14 +1868,14 @@ static BOOL api_RNetServerGetInfo(int cnum,uint16 vuid, char *param,char *data,
       pstring comment;
       uint32 servertype= lp_default_server_announce();
 
-      strcpy(comment,lp_serverstring());
+      pstrcpy(comment,lp_serverstring());
 
       if ((count=get_server_info(SV_TYPE_ALL,&servers,myworkgroup))>0) {
 	for (i=0;i<count;i++)
 	  if (strequal(servers[i].name,local_machine))
       {
 	    servertype = servers[i].type;
-	    strcpy(comment,servers[i].comment);	    
+	    pstrcpy(comment,servers[i].comment);	    
 	  }
       }
       if (servers) free(servers);
@@ -1998,12 +2167,11 @@ static BOOL api_RNetUserGetInfo(int cnum,uint16 vuid, char *param,char *data,
     /* get NIS home of a previously validated user - simeon */
     user_struct *vuser = get_valid_user_struct(vuid);
     DEBUG(3,("  Username of UID %d is %s\n", vuser->uid, vuser->name));
-#if (defined(NETGROUP) && defined(AUTOMOUNT))
-    DEBUG(3,("  HOMESHR for %s is %s\n", vuser->name, vuser->home_share));
-#endif
 
     *rparam_len = 6;
     *rparam = REALLOC(*rparam,*rparam_len);
+
+    DEBUG(4,("RNetUserGetInfo level=%d\n", uLevel));
   
 	/* check it's a supported variant */
 	if (strcmp(str1,"zWrLh") != 0) return False;
@@ -2029,7 +2197,7 @@ static BOOL api_RNetUserGetInfo(int cnum,uint16 vuid, char *param,char *data,
 	p2 = p + usri11_end;
 
 	memset(p,0,21); 
-	strcpy(p+usri11_name,UserName); /* 21 bytes - user name */
+	fstrcpy(p+usri11_name,UserName); /* 21 bytes - user name */
 
 	if (uLevel > 0)
 	{
@@ -2046,9 +2214,9 @@ static BOOL api_RNetUserGetInfo(int cnum,uint16 vuid, char *param,char *data,
 		strcpy(p2,"UserComment");
 		p2 = skip_string(p2,1);
 
-        /* EEK! the cifsrap.txt doesn't have this in!!!! */
+		/* EEK! the cifsrap.txt doesn't have this in!!!! */
 		SIVAL(p,usri11_full_name,PTR_DIFF(p2,p)); /* full name */
-        strcpy(p2,vuser->real_name);	/* simeon */
+		strcpy(p2,vuser->real_name);	/* simeon */
 		p2 = skip_string(p2,1);
 	}
 
@@ -2056,29 +2224,17 @@ static BOOL api_RNetUserGetInfo(int cnum,uint16 vuid, char *param,char *data,
 	{         
 		SSVAL(p,usri11_priv,Connections[cnum].admin_user?USER_PRIV_ADMIN:USER_PRIV_USER); 
 		SIVAL(p,usri11_auth_flags,AF_OP_PRINT);		/* auth flags */
-		SIVALS(p,usri11_password_age,0xffffffff);		/* password age */
+		SIVALS(p,usri11_password_age,-1);		/* password age */
 		SIVAL(p,usri11_homedir,PTR_DIFF(p2,p)); /* home dir */
-		if (*lp_logon_path())
-		{
-			strcpy(p2,lp_logon_path());
-		}
-		else
-		{
-#if (defined(NETGROUP) && defined(AUTOMOUNT))
-            strcpy(p2, vuser->home_share);
-#else
-			strcpy(p2,"\\\\%L\\%U");
-#endif
-		}
-		standard_sub_basic(p2);
+		strcpy(p2, lp_logon_path());
 		p2 = skip_string(p2,1);
 		SIVAL(p,usri11_parms,PTR_DIFF(p2,p)); /* parms */
 		strcpy(p2,"");
 		p2 = skip_string(p2,1);
 		SIVAL(p,usri11_last_logon,0);		/* last logon */
 		SIVAL(p,usri11_last_logoff,0);		/* last logoff */
-		SSVALS(p,usri11_bad_pw_count,0xffffffff);		/* bad pw counts */
-		SSVALS(p,usri11_num_logons,0xffffffff);		/* num logons */
+		SSVALS(p,usri11_bad_pw_count,-1);	/* bad pw counts */
+		SSVALS(p,usri11_num_logons,-1);		/* num logons */
 		SIVAL(p,usri11_logon_server,PTR_DIFF(p2,p)); /* logon server */
 		strcpy(p2,"\\\\*");
 		p2 = skip_string(p2,1);
@@ -2088,7 +2244,7 @@ static BOOL api_RNetUserGetInfo(int cnum,uint16 vuid, char *param,char *data,
 		strcpy(p2,"");
 		p2 = skip_string(p2,1);
 
-		SIVALS(p,usri11_max_storage,0xffffffff);		/* max storage */
+		SIVALS(p,usri11_max_storage,-1);		/* max storage */
 		SSVAL(p,usri11_units_per_week,168);		/* units per week */
 		SIVAL(p,usri11_logon_hours,PTR_DIFF(p2,p)); /* logon hours */
 
@@ -2106,19 +2262,7 @@ static BOOL api_RNetUserGetInfo(int cnum,uint16 vuid, char *param,char *data,
 		SSVAL(p,42,
 		Connections[cnum].admin_user?USER_PRIV_ADMIN:USER_PRIV_USER);
 		SIVAL(p,44,PTR_DIFF(p2,*rdata)); /* home dir */
-		if (*lp_logon_path())
-		{
-			strcpy(p2,lp_logon_path());
-		}
-		else
-		{
-#if (defined(NETGROUP) && defined(AUTOMOUNT))
-            strcpy(p2, vuser->home_share);
-#else
-			strcpy(p2,"\\\\%L\\%U");
-#endif
-		}
-		standard_sub_basic(p2);
+		strcpy(p2,lp_logon_path());
 		p2 = skip_string(p2,1);
 		SIVAL(p,48,PTR_DIFF(p2,*rdata)); /* comment */
 		*p2++ = 0;
@@ -2222,6 +2366,7 @@ static BOOL api_WWkstaUserLogon(int cnum,uint16 vuid, char *param,char *data,
   int uLevel;
   struct pack_desc desc;
   char* name;
+  char* logon_script;
 
   uLevel = SVAL(p,0);
   name = p + 2;
@@ -2264,7 +2409,14 @@ static BOOL api_WWkstaUserLogon(int cnum,uint16 vuid, char *param,char *data,
       PACKS(&desc,"z",mypath); /* computer */
     }
     PACKS(&desc,"z",myworkgroup);/* domain */
-    PACKS(&desc,"z",lp_logon_script());		/* script path */
+
+/* JHT - By calling lp_logon_script() and standard_sub() we have */
+/* made sure all macros are fully substituted and available */
+    logon_script = lp_logon_script();
+    standard_sub( cnum, logon_script );
+    PACKS(&desc,"z", logon_script);		/* script path */
+/* End of JHT mods */
+
     PACKI(&desc,"D",0x00000000);		/* reserved */
   }
 
@@ -2319,7 +2471,6 @@ static BOOL api_WPrintJobGetInfo(int cnum,uint16 vuid, char *param,char *data,
   char *str1 = param+2;
   char *str2 = skip_string(str1,1);
   char *p = skip_string(str2,1);
-  int uJobId = SVAL(p,0);
   int uLevel,cbBuf;
   int count;
   int i;
@@ -2335,20 +2486,19 @@ static BOOL api_WPrintJobGetInfo(int cnum,uint16 vuid, char *param,char *data,
   bzero(&desc,sizeof(desc));
   bzero(&status,sizeof(status));
 
-  DEBUG(3,("WPrintJobGetInfo uLevel=%d uJobId=0x%X\n",uLevel,uJobId));
+  DEBUG(3,("WPrintJobGetInfo uLevel=%d uJobId=0x%X\n",uLevel,SVAL(p,0)));
 
   /* check it's a supported varient */
   if (strcmp(str1,"WWrLh") != 0) return False;
   if (!check_printjob_info(&desc,uLevel,str2)) return False;
 
-  snum = (unsigned int)uJobId >> 8; /*## valid serice number??*/
-  job = uJobId & 0xFF;
+  printjob_decode(SVAL(p,0), &snum, &job);
 
   if (snum < 0 || !VALID_SNUM(snum)) return(False);
 
   count = get_printqueue(snum,cnum,&queue,&status);
   for (i = 0; i < count; i++) {
-    if ((queue[i].job % 0xFF) == job) break;
+    if ((queue[i].job & 0xFF) == job) break;
   }
   if (mdrcnt > 0) *rdata = REALLOC(*rdata,mdrcnt);
   desc.base = *rdata;
@@ -2466,7 +2616,8 @@ static void fill_printdest_info(int cnum, int snum, int uLevel,
 				struct pack_desc* desc)
 {
   char buf[100];
-  strcpy(buf,SERVICE(snum));
+  strncpy(buf,SERVICE(snum),sizeof(buf)-1);
+  buf[sizeof(buf)-1] = 0;
   strupper(buf);
   if (uLevel <= 1) {
     PACKS(desc,"B9",buf);	/* szName */
@@ -2753,14 +2904,25 @@ static BOOL api_WPrintPortEnum(int cnum,uint16 vuid, char *param,char *data,
 struct
 {
   char * name;
-  char * pipename;
+  char * pipe_clnt_name;
+#ifdef NTDOMAIN
+  char * pipe_srv_name;
+#endif
   int subcommand;
   BOOL (*fn) ();
 } api_fd_commands [] =
   {
-    { "SetNmdPpHndState",	"lsarpc",	1,	api_LsarpcSNPHS },
-    { "TransactNmPipe",	"lsarpc",	0x26,	api_LsarpcTNP },
+#ifdef NTDOMAIN
+    { "TransactNmPipe",     "lsarpc",	"lsass",	0x26,	api_ntLsarpcTNP },
+    { "TransactNmPipe",     "samr",	"lsass",	0x26,	api_samrTNP },
+    { "TransactNmPipe",     "srvsvc",	"lsass",	0x26,	api_srvsvcTNP },
+    { "TransactNmPipe",     "wkssvc",	"ntsvcs",	0x26,	api_wkssvcTNP },
+    { "TransactNmPipe",     "NETLOGON",	"NETLOGON",	0x26,	api_netlogrpcTNP },
+    { NULL,		            NULL,       NULL,	-1,	(BOOL (*)())api_Unsupported }
+#else
+    { "TransactNmPipe"  ,	"lsarpc",	0x26,	api_LsarpcTNP },
     { NULL,		NULL,		-1,	(BOOL (*)())api_Unsupported }
+#endif
   };
 
 /****************************************************************************
@@ -2774,11 +2936,17 @@ static int api_fd_reply(int cnum,uint16 vuid,char *outbuf,
   char *rparam = NULL;
   int rdata_len = 0;
   int rparam_len = 0;
-  BOOL reply=False;
+
+  BOOL reply    = False;
+  BOOL bind_req = False;
+  BOOL set_nphs = False;
+
   int i;
   int fd;
   int subcommand;
+  char *pipe_name;
   
+  DEBUG(5,("api_fd_reply\n"));
   /* First find out the name of this file. */
   if (suwcnt != 2)
     {
@@ -2789,46 +2957,113 @@ static int api_fd_reply(int cnum,uint16 vuid,char *outbuf,
   /* Get the file handle and hence the file name. */
   fd = setup[1];
   subcommand = setup[0];
+  pipe_name = get_rpc_pipe_hnd_name(fd);
+
+  if (pipe_name == NULL)
+  {
+    DEBUG(1,("api_fd_reply: INVALID PIPE HANDLE: %x\n", fd));
+  }
+
+  DEBUG(3,("Got API command %d on pipe %s (fd %x)",
+            subcommand, pipe_name, fd));
+  DEBUG(3,("(tdscnt=%d,tpscnt=%d,mdrcnt=%d,mprcnt=%d,cnum=%d,vuid=%d)\n",
+	   tdscnt,tpscnt,mdrcnt,mprcnt,cnum,vuid));
   
-  DEBUG(3,("Got API command %d on pipe %s ",subcommand,Files[fd].name));
-  DEBUG(3,("(tdscnt=%d,tpscnt=%d,mdrcnt=%d,mprcnt=%d)\n",
-	   tdscnt,tpscnt,mdrcnt,mprcnt));
+  for (i = 0; api_fd_commands[i].name; i++)
+  {
+    if (strequal(api_fd_commands[i].pipe_clnt_name, pipe_name) &&
+	    api_fd_commands[i].subcommand == subcommand &&
+	    api_fd_commands[i].fn)
+    {
+	  DEBUG(3,("Doing %s\n", api_fd_commands[i].name));
+	  break;
+    }
+  }
   
-  for (i=0;api_fd_commands[i].name;i++)
-    if (strequal(api_fd_commands[i].pipename, Files[fd].name) &&
-	api_fd_commands[i].subcommand == subcommand &&
-	api_fd_commands[i].fn)
-      {
-	DEBUG(3,("Doing %s\n",api_fd_commands[i].name));
-	break;
-      }
-  
-  rdata = (char *)malloc(1024); if (rdata) bzero(rdata,1024);
+  rdata  = (char *)malloc(1024); if (rdata ) bzero(rdata ,1024);
   rparam = (char *)malloc(1024); if (rparam) bzero(rparam,1024);
   
-  reply = api_fd_commands[i].fn(cnum,vuid,params,data,mdrcnt,mprcnt,
-			        &rdata,&rparam,&rdata_len,&rparam_len);
-  
-  if (rdata_len > mdrcnt ||
-      rparam_len > mprcnt)
+#ifdef NTDOMAIN
+  /* RPC Pipe command 0x26. */
+  if (data != NULL && api_fd_commands[i].subcommand == 0x26)
+  {
+    RPC_HDR hdr;
+
+    /* process the rpc header */
+    char *q = smb_io_rpc_hdr(True, &hdr, data, data, 4, 0);
+    
+	/* bind request received */
+    if ((bind_req = ((q != NULL) && (hdr.pkt_type == RPC_BIND))))
     {
-      reply = api_TooSmall(cnum,vuid,params,data,mdrcnt,mprcnt,
-			   &rdata,&rparam,&rdata_len,&rparam_len);
+      RPC_HDR_RB hdr_rb;
+
+      /* decode the bind request */
+      char *p = smb_io_rpc_hdr_rb(True, &hdr_rb, q, data, 4, 0);
+
+      if ((bind_req = (p != NULL)))
+      {
+        RPC_HDR_BA hdr_ba;
+        fstring ack_pipe_name;
+
+        /* name has to be \PIPE\xxxxx */
+        strcpy(ack_pipe_name, "\\PIPE\\");
+        strcat(ack_pipe_name, api_fd_commands[i].pipe_srv_name);
+
+        /* make a bind acknowledgement */
+        make_rpc_hdr_ba(&hdr_ba,
+               hdr_rb.bba.max_tsize, hdr_rb.bba.max_rsize, hdr_rb.bba.assoc_gid,
+               ack_pipe_name,
+               0x1, 0x0, 0x0,
+               &(hdr_rb.transfer));
+
+        p = smb_io_rpc_hdr_ba(False, &hdr_ba, rdata + 0x10, rdata, 4, 0);
+
+		rdata_len = PTR_DIFF(p, rdata);
+
+        make_rpc_hdr(&hdr, RPC_BINDACK, 0x0, hdr.call_id, rdata_len);
+
+        p = smb_io_rpc_hdr(False, &hdr, rdata, rdata, 4, 0);
+        
+        reply = (p != NULL);
+      }
     }
-  
+  }
+#endif
+
+  /* Set Named Pipe Handle state */
+  if (subcommand == 0x1)
+  {
+    set_nphs = True;
+    reply = api_LsarpcSNPHS(fd, cnum, params);
+  }
+
+  if (!bind_req && !set_nphs)
+  {
+    DEBUG(10,("calling api_fd_command\n"));
+
+    reply = api_fd_commands[i].fn(cnum,vuid,params,data,mdrcnt,mprcnt,
+			        &rdata,&rparam,&rdata_len,&rparam_len);
+    DEBUG(10,("called api_fd_command\n"));
+  }
+
+  if (rdata_len > mdrcnt || rparam_len > mprcnt)
+  {
+    reply = api_TooSmall(cnum,vuid,params,data,mdrcnt,mprcnt,
+			   &rdata,&rparam,&rdata_len,&rparam_len);
+  }
   
   /* if we get False back then it's actually unsupported */
   if (!reply)
+  {
     api_Unsupported(cnum,vuid,params,data,mdrcnt,mprcnt,
 		    &rdata,&rparam,&rdata_len,&rparam_len);
+  }
   
   /* now send the reply */
   send_trans_reply(outbuf,rdata,rparam,NULL,rdata_len,rparam_len,0);
   
-  if (rdata)
-    free(rdata);
-  if (rparam)
-    free(rparam);
+  if (rdata ) free(rdata );
+  if (rparam) free(rparam);
   
   return(-1);
 }
@@ -2981,25 +3216,31 @@ static int named_pipe(int cnum,uint16 vuid, char *outbuf,char *name,
 		      int suwcnt,int tdscnt,int tpscnt,
 		      int msrcnt,int mdrcnt,int mprcnt)
 {
+	DEBUG(3,("named pipe command on <%s> name\n", name));
 
-  if (strequal(name,"LANMAN"))
-    return(api_reply(cnum,vuid,outbuf,data,params,tdscnt,tpscnt,mdrcnt,mprcnt));
+	if (strequal(name,"LANMAN"))
+	{
+		return api_reply(cnum,vuid,outbuf,data,params,tdscnt,tpscnt,mdrcnt,mprcnt);
+	}
 
-if (strlen(name) < 1)
-  return(api_fd_reply(cnum,vuid,outbuf,setup,data,params,suwcnt,tdscnt,tpscnt,mdrcnt,mprcnt));
+	if (strlen(name) < 1)
+	{
+		return api_fd_reply(cnum,vuid,outbuf,setup,data,params,suwcnt,tdscnt,tpscnt,mdrcnt,mprcnt);
+	}
 
+	if (setup)
+	{
+		DEBUG(3,("unknown named pipe: setup 0x%X setup1=%d\n", (int)setup[0],(int)setup[1]));
+	}
 
-  DEBUG(3,("named pipe command on <%s> 0x%X setup1=%d\n",
-	   name,(int)setup[0],(int)setup[1]));
-  
-  return(0);
+	return 0;
 }
 
 
 /****************************************************************************
   reply to a SMBtrans
   ****************************************************************************/
-int reply_trans(char *inbuf,char *outbuf)
+int reply_trans(char *inbuf,char *outbuf, int size, int bufsize)
 {
   fstring name;
 
@@ -3023,7 +3264,12 @@ int reply_trans(char *inbuf,char *outbuf)
   int dsoff = SVAL(inbuf,smb_vwv12);
   int suwcnt = CVAL(inbuf,smb_vwv13);
 
-  StrnCpy(name,smb_buf(inbuf),sizeof(name)-1);
+  bzero(name, sizeof(name));
+  fstrcpy(name,smb_buf(inbuf));
+
+  if (dscnt > tdscnt || pscnt > tpscnt) {
+	  exit_server("invalid trans parameters\n");
+  }
   
   if (tdscnt)
     {
@@ -3057,12 +3303,18 @@ int reply_trans(char *inbuf,char *outbuf)
   /* receive the rest of the trans packet */
   while (pscnt < tpscnt || dscnt < tdscnt)
     {
+      BOOL ret;
       int pcnt,poff,dcnt,doff,pdisp,ddisp;
       
-      if (!receive_smb(Client,inbuf, SMB_SECONDARY_WAIT) ||
-	  CVAL(inbuf, smb_com) != SMBtrans)
+      ret = receive_next_smb(Client,oplock_sock,inbuf,bufsize,SMB_SECONDARY_WAIT);
+
+      if ((ret && (CVAL(inbuf, smb_com) != SMBtrans)) || !ret)
 	{
-	  DEBUG(2,("Invalid secondary trans2 packet\n"));
+          if(ret)
+            DEBUG(0,("reply_trans: Invalid secondary trans packet\n"));
+          else
+            DEBUG(0,("reply_trans: %s in getting secondary trans response.\n",
+              (smb_read_error == READ_ERROR) ? "error" : "timeout" ));
 	  if (params) free(params);
 	  if (data) free(data);
 	  if (setup) free(setup);
@@ -3085,6 +3337,10 @@ int reply_trans(char *inbuf,char *outbuf)
       pscnt += pcnt;
       dscnt += dcnt;
 
+      if (dscnt > tdscnt || pscnt > tpscnt) {
+	      exit_server("invalid trans parameters\n");
+      }
+
       if (pcnt)
 	memcpy(params+pdisp,smb_base(inbuf)+poff,pcnt);
       if (dcnt)
@@ -3093,11 +3349,18 @@ int reply_trans(char *inbuf,char *outbuf)
 
 
   DEBUG(3,("trans <%s> data=%d params=%d setup=%d\n",name,tdscnt,tpscnt,suwcnt));
-  
 
   if (strncmp(name,"\\PIPE\\",strlen("\\PIPE\\")) == 0)
+  {
+    DEBUG(5,("calling named_pipe\n"));
     outsize = named_pipe(cnum,vuid,outbuf,name+strlen("\\PIPE\\"),setup,data,params,
 			 suwcnt,tdscnt,tpscnt,msrcnt,mdrcnt,mprcnt);
+  }
+  else
+  {
+    DEBUG(3,("invalid pipe name\n"));
+    outsize = 0;
+  }
 
 
   if (data) free(data);
