@@ -2,7 +2,7 @@
    Unix SMB/Netbios implementation.
    Version 1.9.
    Kanji Extensions
-   Copyright (C) Andrew Tridgell 1992-1997
+   Copyright (C) Andrew Tridgell 1992-1998
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,18 +23,14 @@
      and add all jis codes sequence type at 1995.8.16
      Notes: Hexadecimal code by <ohki@gssm.otuka.tsukuba.ac.jp>
 */
-#ifdef KANJI
 
 #define _KANJI_C_
 #include "includes.h"
 
-/* coding system keep in */
-int coding_system = SJIS_CODE;
-
 /* jis si/so sequence */
-char jis_kso = JIS_KSO;
-char jis_ksi = JIS_KSI;
-char hex_tag = HEXTAG;
+static char jis_kso = JIS_KSO;
+static char jis_ksi = JIS_KSI;
+static char hex_tag = HEXTAG;
 
 /*******************************************************************
   SHIFT JIS functions
@@ -43,9 +39,11 @@ char hex_tag = HEXTAG;
  search token from S1 separated any char of S2
  S1 contain SHIFT JIS chars.
 ********************************************************************/
-char *
-sj_strtok (char *s1, const char *s2)
+char *sj_strtok(char *s1, char *s2)
 {
+  if (lp_client_code_page() != KANJI_CODEPAGE) {
+   return strtok(s1, s2);
+  } else {
     static char *s = NULL;
     char *q;
     if (!s1) {
@@ -77,16 +75,19 @@ sj_strtok (char *s1, const char *s2)
 	return q;
     }
     return NULL;
+  }
 }
 
 /*******************************************************************
  search string S2 from S1
  S1 contain SHIFT JIS chars.
 ********************************************************************/
-char *
-sj_strstr (const char *s1, const char *s2)
+char *sj_strstr(char *s1, char *s2)
 {
-    register int len = strlen ((char *) s2);
+  if (lp_client_code_page() != KANJI_CODEPAGE) {
+    return strstr(s1, s2);
+  } else {
+    int len = strlen ((char *) s2);
     if (!*s2) 
 	return (char *) s1;
     for (;*s1;) {
@@ -101,15 +102,18 @@ sj_strstr (const char *s1, const char *s2)
 	}
     }
     return 0;
+  }
 }
 
 /*******************************************************************
  Search char C from beginning of S.
  S contain SHIFT JIS chars.
 ********************************************************************/
-char *
-sj_strchr (const char *s, int c)
+char *sj_strchr (char *s, int c)
 {
+  if (lp_client_code_page() != KANJI_CODEPAGE) {
+    return strchr(s, c);
+  } else {
     for (; *s; ) {
 	if (*s == c)
 	    return (char *) s;
@@ -120,16 +124,19 @@ sj_strchr (const char *s, int c)
 	}
     }
     return 0;
+  }
 }
 
 /*******************************************************************
  Search char C end of S.
  S contain SHIFT JIS chars.
 ********************************************************************/
-char *
-sj_strrchr (const char *s, int c)
+char *sj_strrchr(char *s, int c)
 {
-    register char *q;
+  if (lp_client_code_page() != KANJI_CODEPAGE) {
+    return strrchr(s, c);
+  } else {
+    char *q;
 
     for (q = 0; *s; ) {
 	if (*s == c) {
@@ -142,6 +149,7 @@ sj_strrchr (const char *s, int c)
 	}
     }
     return q;
+  }
 }
 
 /*******************************************************************
@@ -153,8 +161,7 @@ static char cvtbuf[1024];
 /*******************************************************************
   EUC <-> SJIS
 ********************************************************************/
-static int
-euc2sjis (register int hi, register int lo)
+static int euc2sjis (int hi, int lo)
 {
     if (hi & 1)
 	return ((hi / 2 + (hi < 0xdf ? 0x31 : 0x71)) << 8) |
@@ -163,8 +170,7 @@ euc2sjis (register int hi, register int lo)
 	return ((hi / 2 + (hi < 0xdf ? 0x30 : 0x70)) << 8) | (lo - 2);
 }
 
-static int
-sjis2euc (register int hi, register int lo)
+static int sjis2euc (int hi, int lo)
 {
     if (lo >= 0x9f)
 	return ((hi * 2 - (hi >= 0xe0 ? 0xe0 : 0x60)) << 8) | (lo + 2);
@@ -177,10 +183,9 @@ sjis2euc (register int hi, register int lo)
  Convert FROM contain SHIFT JIS codes to EUC codes
  return converted buffer
 ********************************************************************/
-static char *
-sj_to_euc (const char *from, BOOL overwrite)
+static char *sj_to_euc(char *from, BOOL overwrite)
 {
-    register char *out;
+    char *out;
     char *save;
 
     save = (char *) from;
@@ -210,10 +215,9 @@ sj_to_euc (const char *from, BOOL overwrite)
  Convert FROM contain EUC codes to SHIFT JIS codes
  return converted buffer
 ********************************************************************/
-static char *
-euc_to_sj (const char *from, BOOL overwrite)
+static char *euc_to_sj(char *from, BOOL overwrite)
 {
-    register char *out;
+    char *out;
     char *save;
 
     save = (char *) from;
@@ -242,8 +246,7 @@ euc_to_sj (const char *from, BOOL overwrite)
 /*******************************************************************
   JIS7,JIS8,JUNET <-> SJIS
 ********************************************************************/
-static int
-sjis2jis (register int hi, register int lo)
+static int sjis2jis(int hi, int lo)
 {
     if (lo >= 0x9f)
 	return ((hi * 2 - (hi >= 0xe0 ? 0x160 : 0xe0)) << 8) | (lo - 0x7e);
@@ -252,8 +255,7 @@ sjis2jis (register int hi, register int lo)
 	    (lo - (lo >= 0x7f ? 0x20 : 0x1f));
 }
 
-static int
-jis2sjis (register int hi, register int lo)
+static int jis2sjis(int hi, int lo)
 {
     if (hi & 1)
 	return ((hi / 2 + (hi < 0x5f ? 0x71 : 0xb1)) << 8) |
@@ -266,11 +268,10 @@ jis2sjis (register int hi, register int lo)
  Convert FROM contain JIS codes to SHIFT JIS codes
  return converted buffer
 ********************************************************************/
-static char *
-jis8_to_sj (const char *from, BOOL overwrite)
+static char *jis8_to_sj(char *from, BOOL overwrite)
 {
-    register char *out;
-    register int shifted;
+    char *out;
+    int shifted;
     char *save;
 
     shifted = _KJ_ROMAN;
@@ -317,11 +318,10 @@ jis8_to_sj (const char *from, BOOL overwrite)
  Convert FROM contain SHIFT JIS codes to JIS codes
  return converted buffer
 ********************************************************************/
-static char *
-sj_to_jis8 (const char *from, BOOL overwrite)
+static char *sj_to_jis8(char *from, BOOL overwrite)
 {
-    register char *out;
-    register int shifted;
+    char *out;
+    int shifted;
     char *save;
 
     shifted = _KJ_ROMAN;
@@ -374,11 +374,10 @@ sj_to_jis8 (const char *from, BOOL overwrite)
  Convert FROM contain 7 bits JIS codes to SHIFT JIS codes
  return converted buffer
 ********************************************************************/
-static char *
-jis7_to_sj (const char *from, BOOL overwrite)
+static char *jis7_to_sj(char *from, BOOL overwrite)
 {
-    register char *out;
-    register int shifted;
+    char *out;
+    int shifted;
     char *save;
 
     shifted = _KJ_ROMAN;
@@ -434,11 +433,10 @@ jis7_to_sj (const char *from, BOOL overwrite)
  Convert FROM contain SHIFT JIS codes to 7 bits JIS codes
  return converted buffer
 ********************************************************************/
-static char *
-sj_to_jis7 (const char *from, BOOL overwrite)
+static char *sj_to_jis7(char *from, BOOL overwrite)
 {
-    register char *out;
-    register int shifted;
+    char *out;
+    int shifted;
     char *save;
 
     shifted = _KJ_ROMAN;
@@ -511,11 +509,10 @@ sj_to_jis7 (const char *from, BOOL overwrite)
  Convert FROM contain 7 bits JIS(junet) codes to SHIFT JIS codes
  return converted buffer
 ********************************************************************/
-static char *
-junet_to_sj (const char *from, BOOL overwrite)
+static char *junet_to_sj(char *from, BOOL overwrite)
 {
-    register char *out;
-    register int shifted;
+    char *out;
+    int shifted;
     char *save;
 
     shifted = _KJ_ROMAN;
@@ -568,11 +565,10 @@ junet_to_sj (const char *from, BOOL overwrite)
  Convert FROM contain SHIFT JIS codes to 7 bits JIS(junet) codes
  return converted buffer
 ********************************************************************/
-static char *
-sj_to_junet (const char *from, BOOL overwrite)
+static char *sj_to_junet(char *from, BOOL overwrite)
 {
-    register char *out;
-    register int shifted;
+    char *out;
+    int shifted;
     char *save;
 
     shifted = _KJ_ROMAN;
@@ -638,8 +634,7 @@ sj_to_junet (const char *from, BOOL overwrite)
   HEX <-> SJIS
 ********************************************************************/
 /* ":xx" -> a byte */
-static char *
-hex_to_sj (const char *from, BOOL overwrite)
+static char *hex_to_sj(char *from, BOOL overwrite)
 {
     char *sp, *dp;
     
@@ -664,8 +659,7 @@ hex_to_sj (const char *from, BOOL overwrite)
 /*******************************************************************
   kanji/kana -> ":xx" 
 ********************************************************************/
-static char *
-sj_to_hex (const char *from, BOOL overwrite)
+static char *sj_to_hex(char *from, BOOL overwrite)
 {
     unsigned char *sp, *dp;
     
@@ -699,10 +693,41 @@ sj_to_hex (const char *from, BOOL overwrite)
 }
 
 /*******************************************************************
-  kanji/kana -> ":xx" 
+  CAP <-> SJIS
 ********************************************************************/
-static char *
-sj_to_cap (const char *from, BOOL overwrite)
+/* ":xx" CAP -> a byte */
+static char *cap_to_sj(char *from, BOOL overwrite)
+{
+    char *sp, *dp;
+
+    sp = (char *) from;
+    dp = cvtbuf;
+    while (*sp) {
+        /*
+         * The only change between this and hex_to_sj is here. sj_to_cap only
+         * translates characters greater or equal to 0x80 - make sure that here
+         * we only do the reverse (that's why the strchr is used rather than
+         * isxdigit. Based on fix from ado@elsie.nci.nih.gov (Arthur David Olson).
+         */
+        if (*sp == hex_tag && (strchr ("89abcdefABCDEF", sp[1]) != NULL) && isxdigit (sp[2])) {
+            *dp++ = (hex2bin (sp[1])<<4) | (hex2bin (sp[2]));
+            sp += 3;
+        } else
+            *dp++ = *sp++;
+    }
+    *dp = '\0';
+    if (overwrite) {
+        strcpy ((char *) from, (char *) cvtbuf);
+        return (char *) from;
+    } else {
+        return cvtbuf;
+    }
+}
+
+/*******************************************************************
+  kanji/kana -> ":xx" - CAP format.
+********************************************************************/
+static char *sj_to_cap(char *from, BOOL overwrite)
 {
     unsigned char *sp, *dp;
 
@@ -730,8 +755,7 @@ sj_to_cap (const char *from, BOOL overwrite)
 /*******************************************************************
  sj to sj
 ********************************************************************/
-static char *
-sj_to_sj (const char *from, BOOL overwrite)
+static char *sj_to_sj(char *from, BOOL overwrite)
 {
     if (!overwrite) {
 	strcpy (cvtbuf, (char *) from);
@@ -746,11 +770,10 @@ sj_to_sj (const char *from, BOOL overwrite)
  _dos_to_unix		_unix_to_dos
 ************************************************************************/
 
-char* (*_dos_to_unix) (const char *str, BOOL overwrite) = sj_to_sj;
-char* (*_unix_to_dos) (const char *str, BOOL overwrite) = sj_to_sj;
+char *(*_dos_to_unix)(char *str, BOOL overwrite) = sj_to_sj;
+char *(*_unix_to_dos)(char *str, BOOL overwrite) = sj_to_sj;
 
-static int
-setup_string_function (int codes)
+static int setup_string_function(int codes)
 {
     switch (codes) {
     default:
@@ -787,7 +810,7 @@ setup_string_function (int codes)
 
     case CAP_CODE:
 	_dos_to_unix = sj_to_cap;
-	_unix_to_dos = hex_to_sj;
+	_unix_to_dos = cap_to_sj;
 	break;
     }
     return codes;
@@ -796,9 +819,9 @@ setup_string_function (int codes)
 /*
  * Interpret coding system.
  */
-int interpret_coding_system(char *str, int def)
+int interpret_coding_system(char *str)
 {
-    int codes = def;
+    int codes = UNKNOWN_CODE;
     
     if (strequal (str, "sjis")) {
 	codes = SJIS_CODE;
@@ -888,7 +911,3 @@ int interpret_coding_system(char *str, int def)
     }	
     return setup_string_function (codes);
 }
-#else 
- int kanji_dummy_procedure(void)
-{return 0;}
-#endif /* KANJI */
