@@ -1,3 +1,6 @@
+/* Copyright (C) 1995-1998 Samba-Team */
+/* Copyright (C) 1998 John H Terpstra <jht@aquasoft.com.au> */
+
 /* local definitions for file server */
 #ifndef _LOCAL_H
 #define _LOCAL_H
@@ -17,13 +20,14 @@
    refer to the special "printers" service */
 #define PRINTERS_NAME "printers"
 
-/* This defines the name of the printcap file. It is MOST UNLIKELY that
-   this will change BUT! Specifying a file with the format of a printcap
-   file but containing only a subset of the printers actually in your real 
-   printcap file is a quick-n-dirty way to allow dynamic access to a subset
-   of available printers.
-*/
-#define PRINTCAP_NAME "/etc/printcap"
+/* Yves Gaige <yvesg@hptnodur.grenoble.hp.com> requested this */
+/* Set to 8 if old clients break because of this, the max value is 15 */
+#define MAXPRINTERLEN 15
+
+/* define what facility to use for syslog */
+#ifndef SYSLOG_FACILITY
+#define SYSLOG_FACILITY LOG_DAEMON
+#endif
 
 /* set these to define the limits of the server. NOTE These are on a
    per-client basis. Thus any one machine can't connect to more than
@@ -32,6 +36,11 @@
 #define MAX_CONNECTIONS 127
 #define MAX_OPEN_FILES 100
 
+/* Default size of shared memory used for share mode locking */
+#ifndef SHMEM_SIZE
+#define SHMEM_SIZE (1024*MAX_OPEN_FILES)
+#endif
+
 /* the max number of connections that the smbstatus program will show */
 #define MAXSTATUS 1000
 
@@ -43,7 +52,7 @@
 #define WORDMAX 0xFFFF
 
 /* the maximum password length before we declare a likely attack */
-#define MAX_PASSWORD_LENGTH 200
+#define MAX_PASS_LEN 200
 
 /* separators for lists */
 #define LIST_SEP " \t,;:\n\r"
@@ -75,6 +84,12 @@
    manager window? */
 #define FSTYPE_STRING "Samba"
 
+
+/* the default guest account - normally set in the Makefile or smb.conf */
+#ifndef GUEST_ACCOUNT
+#define GUEST_ACCOUNT "nobody"
+#endif
+
 /* do you want smbd to send a 1 byte packet to nmbd to trigger it to start 
    when smbd starts? */
 #ifndef PRIME_NMBD
@@ -147,10 +162,6 @@
 /* shall we support browse requests via a FIFO to nmbd? */
 #define ENABLE_FIFO 1
 
-/* keep the password server open, this uses up a aocket, but is needed
-   by many apps */
-#define KEEP_PASSWORD_SERVER_OPEN 1
-
 /* how long to wait for a socket connect to happen */
 #define LONG_CONNECT_TIMEOUT 30
 #define SHORT_CONNECT_TIMEOUT 5
@@ -159,4 +170,18 @@
 /* the directory to sit in when idle */
 /* #define IDLE_DIR "/" */
 
+/* Timout (in seconds) to wait for an oplock break
+   message to return from the client. */
+
+#define OPLOCK_BREAK_TIMEOUT 30
+
+/* Timout (in seconds) to add to the oplock break timeout
+   to wait for the smbd to smbd message to return. */
+
+#define OPLOCK_BREAK_TIMEOUT_FUDGEFACTOR 2
+
+/* the read preciction code has been disabled until some problems with
+   it are worked out */
+#define USE_READ_PREDICTION 0
+
 #endif
