@@ -2,7 +2,7 @@
    Unix SMB/Netbios implementation.
    Version 1.9.
    file read prediction routines
-   Copyright (C) Andrew Tridgell 1992-1997
+   Copyright (C) Andrew Tridgell 1992-1998
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 
 extern int DEBUGLEVEL;
 
+#if USE_READ_PREDICTION
 
 /* variables used by the read prediction module */
 static int rp_fd = -1;
@@ -36,7 +37,7 @@ static int rp_timeout = 5;
 static time_t rp_time = 0;
 static char *rp_buffer = NULL;
 static BOOL predict_skip=False;
-time_t smb_last_time=(time_t)0;
+extern time_t smb_last_time;
 
 /****************************************************************************
 handle read prediction on a file
@@ -53,24 +54,37 @@ int read_predict(int fd,int offset,char *buf,char **ptr,int num)
       offset >= rp_offset && 
       possible>0 &&
       smb_last_time-rp_time < rp_timeout)
-    {
-      ret = possible;
-      if (buf)
-	memcpy(buf,rp_buffer + (offset-rp_offset),possible);
-      else
-	*ptr = rp_buffer + (offset-rp_offset);
-      DEBUG(5,("read-prediction gave %d bytes of %d\n",ret,num));
-    }
+  {
+    ret = possible;
+    if (buf)
+      memcpy(buf,rp_buffer + (offset-rp_offset),possible);
+    else
+      *ptr = rp_buffer + (offset-rp_offset);
+    DEBUG(5,("read-prediction gave %d bytes of %d\n",ret,num));
+  }
 
   if (ret == num) {
     predict_skip = True;
   } else {
-    predict_skip = False;
+    struct stat rp_stat;
 
-    /* prepare the next prediction */
-    rp_predict_fd = fd;
-    rp_predict_offset = offset + num;
-    rp_predict_length = num;
+    /* Find the end of the file - ensure we don't
+       read predict beyond it. */
+    if(fstat(fd,&rp_stat) < 0)
+    {
+      DEBUG(0,("read-prediction failed on fstat. Error was %s\n", strerror(errno)));
+      predict_skip = True;
+    }
+    else
+    {
+      predict_skip = False;
+
+      /* prepare the next prediction */
+      rp_predict_fd = fd;
+      /* Make sure we don't seek beyond the end of the file. */
+      rp_predict_offset = MIN((offset + num),rp_stat.st_size);
+      rp_predict_length = num;
+    }
   }
 
   if (ret < 0) ret = 0;
@@ -81,7 +95,7 @@ int read_predict(int fd,int offset,char *buf,char **ptr,int num)
 /****************************************************************************
 pre-read some data
 ****************************************************************************/
-void do_read_prediction()
+void do_read_prediction(void)
 {
   static int readsize = 0;
 
@@ -143,3 +157,6 @@ void invalidate_read_prediction(int fd)
    rp_predict_fd = -1;
 }
 
+#else
+ void read_prediction_dummy(void) ;
+#endif
