@@ -2,7 +2,7 @@
    Unix SMB/Netbios implementation.
    Version 1.9.
    Samba system utilities
-   Copyright (C) Andrew Tridgell 1992-1997
+   Copyright (C) Andrew Tridgell 1992-1998
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -194,6 +194,15 @@ now for utime()
 ********************************************************************/
 int sys_utime(char *fname,struct utimbuf *times)
 {
+  /* if the modtime is 0 or -1 then ignore the call and
+     return success */
+  if (times->modtime == (time_t)0 || times->modtime == (time_t)-1)
+    return 0;
+  
+  /* if the access time is 0 or -1 then set it to the modtime */
+  if (times->actime == (time_t)0 || times->actime == (time_t)-1)
+    times->actime = times->modtime;
+   
   return(utime(dos_to_unix(fname,False),times));
 }
 
@@ -202,8 +211,7 @@ for rename across filesystems Patch from Warren Birnbaum
 <warrenb@hpcvscdp.cv.hp.com>
 **********************************************************/
 
-static int
-copy_reg (const char *source, const char *dest)
+static int copy_reg(char *source, const char *dest)
 {
   struct stat source_stats;
   int ifd;
@@ -311,8 +319,8 @@ int sys_rename(char *from, char *to)
     int rcode;  
     pstring zfrom, zto;
 
-    strcpy (zfrom, dos_to_unix (from, False));
-    strcpy (zto, dos_to_unix (to, False));
+    pstrcpy (zfrom, dos_to_unix (from, False));
+    pstrcpy (zto, dos_to_unix (to, False));
     rcode = rename (zfrom, zto);
 
     if (errno == EXDEV) 
