@@ -2,7 +2,7 @@
    Unix SMB/Netbios implementation.
    Version 1.9.
    Test validity of smb.conf
-   Copyright (C) Karl Auer 1993, 1994-1997
+   Copyright (C) Karl Auer 1993, 1994-1998
 
    Extensively modified by Andrew Tridgell, 1995
    
@@ -38,6 +38,7 @@
 /* these live in util.c */
 extern FILE *dbf;
 extern int DEBUGLEVEL;
+extern pstring myhostname;
 
  int main(int argc, char *argv[])
 {
@@ -51,15 +52,21 @@ extern int DEBUGLEVEL;
   charset_initialise();
 
   if (argc < 2)
-    strcpy(configfile,CONFIGFILE);
+    pstrcpy(configfile,CONFIGFILE);
   else
-    strcpy(configfile,argv[1]);
+    pstrcpy(configfile,argv[1]);
 
   dbf = stdout;
   DEBUGLEVEL = 2;
 
   printf("Load smb config files from %s\n",configfile);
 
+  if(!get_myname(myhostname,NULL))
+  {
+    printf("Failed to get my hostname.\n");
+    return(1);
+  }
+
   if (!lp_load(configfile,False))
     {
       printf("Error loading services.\n");
@@ -82,7 +89,7 @@ extern int DEBUGLEVEL;
       printf("Press enter to see a dump of your service definitions\n");
       fflush(stdout);
       getc(stdin);
-      lp_dump();      
+      lp_dump(stdout);      
     }
   
   if (argc == 4)
