@@ -2,7 +2,7 @@
    Unix SMB/Netbios implementation.
    Version 1.9.
    time handling functions
-   Copyright (C) Andrew Tridgell 1992-1997
+   Copyright (C) Andrew Tridgell 1992-1998
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -74,6 +74,7 @@ static int tm_diff(struct tm *a, struct tm *b)
   int hours = 24*days + (a->tm_hour - b->tm_hour);
   int minutes = 60*hours + (a->tm_min - b->tm_min);
   int seconds = 60*minutes + (a->tm_sec - b->tm_sec);
+
   return seconds;
 }
 
@@ -93,6 +94,11 @@ init the time differences
 void TimeInit(void)
 {
   serverzone = TimeZone(time(NULL));
+
+  if ((serverzone % 60) != 0) {
+	  DEBUG(1,("WARNING: Your timezone is not a multiple of 1 minute.\n"));
+  }
+
   DEBUG(4,("Serverzone is %d\n",serverzone));
 }
 
@@ -279,7 +285,7 @@ void put_long_date(char *p,time_t t)
   }
 
   /* this converts GMT to kludge-GMT */
-  t -= TimeDiff(t) - serverzone; 
+  t -= LocTimeDiff(t) - serverzone; 
 
   d = (double) (t);
 
@@ -298,7 +304,7 @@ void put_long_date(char *p,time_t t)
 /****************************************************************************
 check if it's a null mtime
 ****************************************************************************/
-static BOOL null_mtime(time_t mtime)
+BOOL null_mtime(time_t mtime)
 {
   if (mtime == 0 || mtime == 0xFFFFFFFF || mtime == (time_t)-1)
     return(True);
@@ -445,25 +451,6 @@ time_t make_unix_date3(void *date_ptr)
   return(t);
 }
 
-/****************************************************************************
-set the time on a file
-****************************************************************************/
-BOOL set_filetime(char *fname,time_t mtime)
-{  
-  struct utimbuf times;
-
-  if (null_mtime(mtime)) return(True);
-
-  times.modtime = times.actime = mtime;
-
-  if (sys_utime(fname,&times)) {
-    DEBUG(4,("set_filetime(%s) failed: %s\n",fname,strerror(errno)));
-  }
-    
-  return(True);
-}
-
-
 /****************************************************************************
   return the date and time as a string
 ****************************************************************************/
@@ -476,20 +463,44 @@ char *timestring(void )
 #ifdef NO_STRFTIME
   fstrcpy(TimeBuf, asctime(tm));
 #elif defined(CLIX) || defined(CONVEX)
-  strftime(TimeBuf,100,"%m/%d/%y %I:%M:%S %p",tm);
+  strftime(TimeBuf,100,"%m/%d/%Y %I:%M:%S %p",tm);
 #elif defined(AMPM)
-  strftime(TimeBuf,100,"%D %r",tm);
+  strftime(TimeBuf,100,"%m/%d/%Y %r",tm);
 #elif defined(TZ_TIME)
   {
     int zone = TimeDiff(t);
     int absZoneMinutes = (zone<0 ? -zone : zone) / 60;
-    size_t len = strftime(TimeBuf,sizeof(TimeBuf)-6,"%D %T",tm);
+    size_t len = strftime(TimeBuf,sizeof(TimeBuf)-6,"%m/%d/%Y %T",tm);
     sprintf(TimeBuf+len," %c%02d%02d",
 	    zone<0?'+':'-',absZoneMinutes/60,absZoneMinutes%60);
   }
 #else
-  strftime(TimeBuf,100,"%D %T",tm);
+  strftime(TimeBuf,100,"%m/%d/%Y %T",tm);
 #endif
   return(TimeBuf);
 }
 
+/****************************************************************************
+  return the best approximation to a 'create time' under UNIX from a stat
+  structure.
+****************************************************************************/
+
+time_t get_create_time(struct stat *st,BOOL fake_dirs)
+{
+  time_t ret, ret1;
+
+  if(S_ISDIR(st->st_mode) && fake_dirs)
+    return (time_t)315493200L;          /* 1/1/1980 */
+    
+  ret = MIN(st->st_ctime, st->st_mtime);
+  ret1 = MIN(ret, st->st_atime);
+
+  if(ret1 != (time_t)0)
+    return ret1;
+
+  /*
+   * One of ctime, mtime or atime was zero (probably atime).
+   * Just return MIN(ctime, mtime).
+   */
+  return ret;
+}
