@@ -2,7 +2,7 @@
    Unix SMB/Netbios implementation.
    Version 1.9.
    SMB transaction2 handling
-   Copyright (C) Jeremy Allison 1994-1997
+   Copyright (C) Jeremy Allison 1994-1998
 
    Extensively modified by Andrew Tridgell, 1995
 
@@ -30,6 +30,10 @@ extern connection_struct Connections[];
 extern files_struct Files[];
 extern BOOL case_sensitive;
 extern int Client;
+extern int oplock_sock;
+extern int smb_read_error;
+extern fstring local_machine;
+extern int global_oplock_break;
 
 /****************************************************************************
   Send the required number of replies back.
@@ -53,7 +57,8 @@ static int send_trans2_replies(char *outbuf, int bufsize, char *params,
   char *pp = params;
   char *pd = pdata;
   int params_sent_thistime, data_sent_thistime, total_sent_thistime;
-  int alignment_offset = 1;
+  int alignment_offset = 3;
+  int data_alignment_offset = 0;
 
   /* Initially set the wcnt area to be 10 - this is true for all
      trans2 replies */
@@ -67,18 +72,30 @@ static int send_trans2_replies(char *outbuf, int bufsize, char *params,
       return 0;
     }
 
+  /* When sending params and data ensure that both are nicely aligned */
+  /* Only do this alignment when there is also data to send - else
+     can cause NT redirector problems. */
+  if (((params_to_send % 4) != 0) && (data_to_send != 0))
+	  data_alignment_offset = 4 - (params_to_send % 4);
+
   /* Space is bufsize minus Netbios over TCP header minus SMB header */
-  /* The alignment_offset is to align the param and data bytes on an even byte
+  /* The alignment_offset is to align the param bytes on an even byte
      boundary. NT 4.0 Beta needs this to work correctly. */
-  useable_space = bufsize - ((smb_buf(outbuf)+alignment_offset) - outbuf);
+  useable_space = bufsize - ((smb_buf(outbuf)+
+			      alignment_offset+data_alignment_offset) - 
+			     outbuf);
+
   /* useable_space can never be more than max_send minus the
      alignment offset. */
-  useable_space = MIN(useable_space, max_send - alignment_offset);
+  useable_space = MIN(useable_space, 
+		      max_send - (alignment_offset+data_alignment_offset));
 
-  while( params_to_send || data_to_send)
+
+  while (params_to_send || data_to_send)
     {
       /* Calculate whether we will totally or partially fill this packet */
-      total_sent_thistime = params_to_send + data_to_send + alignment_offset;
+      total_sent_thistime = params_to_send + data_to_send + 
+	      alignment_offset + data_alignment_offset;
       /* We can never send more than useable_space */
       total_sent_thistime = MIN(total_sent_thistime, useable_space);
 
@@ -119,7 +136,8 @@ static int send_trans2_replies(char *outbuf, int bufsize, char *params,
 	  /* The offset of the data bytes is the offset of the
 	     parameter bytes plus the number of parameters being sent this time */
 	  SSVAL(outbuf,smb_droff,((smb_buf(outbuf)+alignment_offset) - 
-				  smb_base(outbuf)) + params_sent_thistime);
+				  smb_base(outbuf)) + 
+		params_sent_thistime + data_alignment_offset);
 	  SSVAL(outbuf,smb_drdisp, pd - pdata);
 	}
 
@@ -128,7 +146,7 @@ static int send_trans2_replies(char *outbuf, int bufsize, char *params,
 	memcpy((smb_buf(outbuf)+alignment_offset),pp,params_sent_thistime);
       /* Copy in the data bytes */
       if(data_sent_thistime)
-	memcpy(smb_buf(outbuf)+alignment_offset+params_sent_thistime,pd,data_sent_thistime);
+	memcpy(smb_buf(outbuf)+alignment_offset+params_sent_thistime+data_alignment_offset,pd,data_sent_thistime);
 
       DEBUG(9,("t2_rep: params_sent_thistime = %d, data_sent_thistime = %d, useable_space = %d\n",
 	       params_sent_thistime, data_sent_thistime, useable_space));
@@ -166,7 +184,7 @@ static int call_trans2open(char *inbuf, char *outbuf, int bufsize, int cnum,
   char *params = *pparams;
   int16 open_mode = SVAL(params, 2);
   int16 open_attr = SVAL(params,6);
-  BOOL oplock_request = BITSETW(params,1);
+  BOOL oplock_request = (((SVAL(params,0)|(1<<1))>>1) | ((SVAL(params,0)|(1<<2))>>1));
 #if 0
   BOOL return_additional_info = BITSETW(params,0);
   int16 open_sattr = SVAL(params, 4);
@@ -206,6 +224,7 @@ static int call_trans2open(char *inbuf, char *outbuf, int bufsize, int cnum,
       unix_ERR_class = ERRDOS;
       unix_ERR_code = ERRbadpath;
     }
+    Files[fnum].reserved = False;
     return(UNIXERROR(ERRDOS,ERRnoaccess));
   }
 
@@ -213,7 +232,7 @@ static int call_trans2open(char *inbuf, char *outbuf, int bufsize, int cnum,
       
       
   open_file_shared(fnum,cnum,fname,open_mode,open_ofun,unixmode,
-		   &rmode,&smb_action);
+		   oplock_request, &rmode,&smb_action);
       
   if (!Files[fnum].open)
   {
@@ -222,11 +241,12 @@ static int call_trans2open(char *inbuf, char *outbuf, int bufsize, int cnum,
       unix_ERR_class = ERRDOS;
       unix_ERR_code = ERRbadpath;
     }
+    Files[fnum].reserved = False;
     return(UNIXERROR(ERRDOS,ERRnoaccess));
   }
 
   if (fstat(Files[fnum].fd_ptr->fd,&sbuf) != 0) {
-    close_file(fnum, 0);
+    close_file(fnum,False);
     return(ERROR(ERRDOS,ERRnoaccess));
   }
     
@@ -235,7 +255,7 @@ static int call_trans2open(char *inbuf, char *outbuf, int bufsize, int cnum,
   mtime = sbuf.st_mtime;
   inode = sbuf.st_ino;
   if (fmode & aDIR) {
-    close_file(fnum, 0);
+    close_file(fnum,False);
     return(ERROR(ERRDOS,ERRnoaccess));
   }
 
@@ -252,7 +272,7 @@ static int call_trans2open(char *inbuf, char *outbuf, int bufsize, int cnum,
   SSVAL(params,12,rmode);
 
   if (oplock_request && lp_fake_oplocks(SNUM(cnum))) {
-    smb_action |= (1<<15);
+    smb_action |= EXTENDED_OPLOCK_GRANTED;
   }
 
   SSVAL(params,18,smb_action);
@@ -282,7 +302,8 @@ static int get_lanman2_dir_entry(int cnum,char *path_mask,int dirtype,int info_l
   pstring fname;
   BOOL matched;
   char *p, *pdata = *ppdata;
-  int reskey=0, prev_dirpos=0;
+  uint32 reskey=0;
+  int prev_dirpos=0;
   int mode=0;
   uint32 size=0,len;
   uint32 mdate=0, adate=0, cdate=0;
@@ -317,9 +338,16 @@ static int get_lanman2_dir_entry(int cnum,char *path_mask,int dirtype,int info_l
       prev_dirpos = TellDir(Connections[cnum].dirptr);
       dname = ReadDirName(Connections[cnum].dirptr);
 
-      reskey = TellDir(Connections[cnum].dirptr);
+      /*
+       * Due to bugs in NT client redirectors we are not using
+       * resume keys any more - set them to zero.
+       * Check out the related comments in findfirst/findnext.
+       * JRA.
+       */
 
-      DEBUG(6,("get_lanman2_dir_entry:readdir on dirptr 0x%x now at offset %d\n",
+      reskey = 0;
+
+      DEBUG(8,("get_lanman2_dir_entry:readdir on dirptr 0x%x now at offset %d\n",
 	       Connections[cnum].dirptr,TellDir(Connections[cnum].dirptr)));
       
       if (!dname) 
@@ -358,7 +386,7 @@ static int get_lanman2_dir_entry(int cnum,char *path_mask,int dirtype,int info_l
 	  size = sbuf.st_size;
 	  mdate = sbuf.st_mtime;
 	  adate = sbuf.st_atime;
-	  cdate = sbuf.st_ctime;
+	  cdate = get_create_time(&sbuf,lp_fake_dir_create_times(SNUM(cnum)));
 	  if(mode & aDIR)
 	    size = 0;
 
@@ -607,6 +635,17 @@ static int call_trans2findfirst(char *inbuf, char *outbuf, int bufsize, int cnum
       unix_ERR_class = ERRDOS;
       unix_ERR_code = ERRbadpath;
     }
+
+#if 0
+    /* Ugly - NT specific hack - maybe not needed ? (JRA) */
+    if((errno == ENOTDIR) && (Protocol >= PROTOCOL_NT1) && 
+       (get_remote_arch() == RA_WINNT))
+    {
+      unix_ERR_class = ERRDOS;
+      unix_ERR_code = ERRbaddirectory;
+    }
+#endif 
+
     return(ERROR(ERRDOS,ERRbadpath));
   }
 
@@ -633,18 +672,7 @@ static int call_trans2findfirst(char *inbuf, char *outbuf, int bufsize, int cnum
 
   dptr_num = dptr_create(cnum,directory, True ,SVAL(inbuf,smb_pid));
   if (dptr_num < 0)
-    {
-      if(dptr_num == -2)
-      {
-        if((errno == ENOENT) && bad_path)
-        {
-          unix_ERR_class = ERRDOS;
-          unix_ERR_code = ERRbadpath;
-        }
-        return (UNIXERROR(ERRDOS,ERRbadpath));
-      }
-      return(ERROR(ERRDOS,ERRbadpath));
-    }
+    return(ERROR(ERRDOS,ERRbadfile));
 
   /* convert the formatted masks */
   {
@@ -724,6 +752,14 @@ static int call_trans2findfirst(char *inbuf, char *outbuf, int bufsize, int cnum
       dptr_num = -1;
     }
 
+  /* 
+   * If there are no matching entries we must return ERRDOS/ERRbadfile - 
+   * from observation of NT.
+   */
+
+  if(numentries == 0)
+    return(ERROR(ERRDOS,ERRbadfile));
+
   /* At this point pdata points to numentries directory entries. */
 
   /* Set up the return parameter block */
@@ -736,7 +772,7 @@ static int call_trans2findfirst(char *inbuf, char *outbuf, int bufsize, int cnum
   send_trans2_replies( outbuf, bufsize, params, 10, pdata, PTR_DIFF(p,pdata));
 
   if ((! *directory) && dptr_path(dptr_num))
-    sprintf(directory,"(%s)",dptr_path(dptr_num));
+    slprintf(directory,sizeof(directory)-1, "(%s)",dptr_path(dptr_num));
 
   DEBUG(4,("%s %s mask=%s directory=%s cnum=%d dirtype=%d numentries=%d\n",
 	timestring(),
@@ -769,6 +805,7 @@ static int call_trans2findnext(char *inbuf, char *outbuf, int length, int bufsiz
   BOOL close_if_end = BITSETW(params+10,1);
   BOOL requires_resume_key = BITSETW(params+10,2);
   BOOL continue_bit = BITSETW(params+10,3);
+  pstring resume_name;
   pstring mask;
   pstring directory;
   char *p;
@@ -780,11 +817,15 @@ static int call_trans2findnext(char *inbuf, char *outbuf, int length, int bufsiz
   BOOL out_of_space = False;
   int space_remaining;
 
-  *mask = *directory = 0;
+  *mask = *directory = *resume_name = 0;
 
-  DEBUG(3,("call_trans2findnext: dirhandle = %d, max_data_bytes = %d, maxentries = %d, close_after_request=%d, close_if_end = %d requires_resume_key = %d resume_key = %d continue=%d level = %d\n",
+  pstrcpy( resume_name, params+12);
+
+  DEBUG(3,("call_trans2findnext: dirhandle = %d, max_data_bytes = %d, maxentries = %d, \
+close_after_request=%d, close_if_end = %d requires_resume_key = %d \
+resume_key = %d resume name = %s continue=%d level = %d\n",
 	   dptr_num, max_data_bytes, maxentries, close_after_request, close_if_end, 
-	   requires_resume_key, resume_key, continue_bit, info_level));
+	   requires_resume_key, resume_key, resume_name, continue_bit, info_level));
 
   switch (info_level) 
     {
@@ -812,7 +853,7 @@ static int call_trans2findnext(char *inbuf, char *outbuf, int length, int bufsiz
     return(ERROR(ERRDOS,ERRnomem));
 
   /* Check that the dptr is valid */
-  if(!(Connections[cnum].dirptr = dptr_fetch_lanman2(params, dptr_num)))
+  if(!(Connections[cnum].dirptr = dptr_fetch_lanman2(dptr_num)))
     return(ERROR(ERRDOS,ERRnofiles));
 
   string_set(&Connections[cnum].dirpath,dptr_path(dptr_num));
@@ -844,9 +885,61 @@ static int call_trans2findnext(char *inbuf, char *outbuf, int length, int bufsiz
   space_remaining = max_data_bytes;
   out_of_space = False;
 
-  /* If we have a resume key - seek to the correct position. */
-  if(requires_resume_key && !continue_bit)
-    SeekDir(Connections[cnum].dirptr, resume_key);
+  /* 
+   * Seek to the correct position. We no longer use the resume key but
+   * depend on the last file name instead.
+   */
+  if(requires_resume_key && *resume_name && !continue_bit)
+  {
+    /*
+     * Fix for NT redirector problem triggered by resume key indexes
+     * changing between directory scans. We now return a resume key of 0
+     * and instead look for the filename to continue from (also given
+     * to us by NT/95/smbfs/smbclient). If no other scans have been done between the
+     * findfirst/findnext (as is usual) then the directory pointer
+     * should already be at the correct place. Check this by scanning
+     * backwards looking for an exact (ie. case sensitive) filename match. 
+     * If we get to the beginning of the directory and haven't found it then scan
+     * forwards again looking for a match. JRA.
+     */
+
+    int current_pos, start_pos;
+    char *dname;
+    void *dirptr = Connections[cnum].dirptr;
+    start_pos = TellDir(dirptr);
+    for(current_pos = start_pos; current_pos >= 0; current_pos--)
+    {
+      DEBUG(7,("call_trans2findnext: seeking to pos %d\n", current_pos));
+
+      SeekDir(dirptr, current_pos);
+      dname = ReadDirName(dirptr);
+      if(dname && strcsequal( resume_name, dname))
+      {
+        SeekDir(dirptr, current_pos+1);
+        DEBUG(7,("call_trans2findnext: got match at pos %d\n", current_pos+1 ));
+        break;
+      }
+    }
+
+    /*
+     * Scan forward from start if not found going backwards.
+     */
+
+    if(current_pos < 0)
+    {
+      DEBUG(7,("call_trans2findnext: notfound: seeking to pos %d\n", start_pos));
+      SeekDir(dirptr, start_pos);
+      for(current_pos = start_pos; (dname = ReadDirName(dirptr)) != NULL; SeekDir(dirptr,++current_pos))
+      {
+        if(strcsequal( resume_name, dname))
+        {
+          SeekDir(dirptr, current_pos+1);
+          DEBUG(7,("call_trans2findnext: got match at pos %d\n", current_pos+1 ));
+          break;
+        }
+      } /* end for */
+    } /* end if current_pos */
+  } /* end if requires_resume_key && !continue_bit */
 
   for (i=0;(i<(int)maxentries) && !finished && !out_of_space ;i++)
     {
@@ -892,7 +985,7 @@ static int call_trans2findnext(char *inbuf, char *outbuf, int length, int bufsiz
   send_trans2_replies( outbuf, bufsize, params, 8, pdata, PTR_DIFF(p,pdata));
 
   if ((! *directory) && dptr_path(dptr_num))
-    sprintf(directory,"(%s)",dptr_path(dptr_num));
+    slprintf(directory,sizeof(directory)-1, "(%s)",dptr_path(dptr_num));
 
   DEBUG(3,("%s %s mask=%s directory=%s cnum=%d dirtype=%d numentries=%d\n",
 	   timestring(),
@@ -914,7 +1007,8 @@ static int call_trans2qfsinfo(char *inbuf, char *outbuf, int length, int bufsize
   int data_len;
   struct stat st;
   char *vname = volume_label(SNUM(cnum));
-  
+  int snum = SNUM(cnum);
+ 
   DEBUG(3,("call_trans2qfsinfo: cnum = %d, level = %d\n", cnum, info_level));
 
   if(sys_stat(".",&st)!=0) {
@@ -945,7 +1039,11 @@ static int call_trans2qfsinfo(char *inbuf, char *outbuf, int length, int bufsize
       /* Return volume name */
       int volname_len = MIN(strlen(vname),11);
       data_len = l2_vol_szVolLabel + volname_len + 1;
-      put_dos_date2(pdata,l2_vol_fdateCreation,st.st_ctime);
+      /* 
+       * Add volume serial number - hash of a combination of
+       * the called hostname and the service name.
+       */
+      SIVAL(pdata,0,str_checksum(lp_servicename(snum)) ^ (str_checksum(local_machine)<<16) );
       SCVAL(pdata,l2_vol_cch,volname_len);
       StrnCpy(pdata+l2_vol_szVolLabel,vname,volname_len);
       DEBUG(5,("call_trans2qfsinfo : time = %x, namelen = %d, name = %s\n",st.st_ctime, volname_len,
@@ -966,6 +1064,11 @@ static int call_trans2qfsinfo(char *inbuf, char *outbuf, int length, int bufsize
       break;
     case SMB_QUERY_FS_VOLUME_INFO:      
       data_len = 18 + 2*strlen(vname);
+      /* 
+       * Add volume serial number - hash of a combination of
+       * the called hostname and the service name.
+       */
+      SIVAL(pdata,8,str_checksum(lp_servicename(snum)) ^ (str_checksum(local_machine)<<16) );
       SIVAL(pdata,12,2*strlen(vname));
       PutUniCode(pdata+18,vname);      
       DEBUG(5,("call_trans2qfsinfo : SMB_QUERY_FS_VOLUME_INFO namelen = %d, vol = %s\n", strlen(vname),
@@ -1102,8 +1205,8 @@ static int call_trans2qfilepathinfo(char *inbuf, char *outbuf, int length,
     case SMB_INFO_STANDARD:
     case SMB_INFO_QUERY_EA_SIZE:
       data_size = (info_level==1?22:26);
-      put_dos_date2(pdata,l1_fdateCreation,sbuf.st_ctime); /* create = inode mod */
-      put_dos_date2(pdata,l1_fdateLastAccess,sbuf.st_atime); /* access time */
+      put_dos_date2(pdata,l1_fdateCreation,get_create_time(&sbuf,lp_fake_dir_create_times(SNUM(cnum))));
+      put_dos_date2(pdata,l1_fdateLastAccess,sbuf.st_atime);
       put_dos_date2(pdata,l1_fdateLastWrite,sbuf.st_mtime); /* write time */
       SIVAL(pdata,l1_cbFile,size);
       SIVAL(pdata,l1_cbFileAlloc,ROUNDUP(size,1024));
@@ -1113,7 +1216,7 @@ static int call_trans2qfilepathinfo(char *inbuf, char *outbuf, int length,
 
     case SMB_INFO_QUERY_EAS_FROM_LIST:
       data_size = 24;
-      put_dos_date2(pdata,0,sbuf.st_ctime); /* create time = inode mod time */
+      put_dos_date2(pdata,0,get_create_time(&sbuf,lp_fake_dir_create_times(SNUM(cnum))));
       put_dos_date2(pdata,4,sbuf.st_atime);
       put_dos_date2(pdata,8,sbuf.st_mtime);
       SIVAL(pdata,12,size);
@@ -1131,14 +1234,17 @@ static int call_trans2qfilepathinfo(char *inbuf, char *outbuf, int length,
 
     case SMB_QUERY_FILE_BASIC_INFO:
       data_size = 36; /* w95 returns 40 bytes not 36 - why ?. */
-      put_long_date(pdata,sbuf.st_ctime); /* create time = inode mod time */
-      put_long_date(pdata+8,sbuf.st_atime); /* access time */
+      put_long_date(pdata,get_create_time(&sbuf,lp_fake_dir_create_times(SNUM(cnum))));
+      put_long_date(pdata+8,sbuf.st_atime);
       put_long_date(pdata+16,sbuf.st_mtime); /* write time */
       put_long_date(pdata+24,sbuf.st_mtime); /* change time */
       SIVAL(pdata,32,mode);
 
       DEBUG(5,("SMB_QFBI - "));
-      DEBUG(5,("create: %s ", ctime(&sbuf.st_ctime)));
+      {
+        time_t create_time = get_create_time(&sbuf,lp_fake_dir_create_times(SNUM(cnum)));
+        DEBUG(5,("create: %s ", ctime(&create_time)));
+      }
       DEBUG(5,("access: %s ", ctime(&sbuf.st_atime)));
       DEBUG(5,("write: %s ", ctime(&sbuf.st_mtime)));
       DEBUG(5,("change: %s ", ctime(&sbuf.st_mtime)));
@@ -1159,12 +1265,32 @@ static int call_trans2qfilepathinfo(char *inbuf, char *outbuf, int length,
       data_size = 4;
       break;
 
-    case SMB_QUERY_FILE_NAME_INFO:
+    /* Get the 8.3 name - used if NT SMB was negotiated. */
     case SMB_QUERY_FILE_ALT_NAME_INFO:
+      {
+        pstring short_name;
+        pstrcpy(short_name,fname);
+        /* Mangle if not already 8.3 */
+        if(!is_8_3(short_name, True))
+        {
+          if(!name_map_mangle(short_name,True,SNUM(cnum)))
+            *short_name = '\0';
+        }
+        strncpy(pdata + 4,short_name,12);
+        (pdata + 4)[12] = 0;
+        strupper(pdata + 4);
+        l = strlen(pdata + 4);
+        data_size = 4 + l;
+        SIVAL(pdata,0,l);
+      }
+      break;
+
+    case SMB_QUERY_FILE_NAME_INFO:
       data_size = 4 + l;
       SIVAL(pdata,0,l);
       pstrcpy(pdata+4,fname);
       break;
+
     case SMB_QUERY_FILE_ALLOCATION_INFO:
     case SMB_QUERY_FILE_END_OF_FILEINFO:
       data_size = 8;
@@ -1172,8 +1298,8 @@ static int call_trans2qfilepathinfo(char *inbuf, char *outbuf, int length,
       break;
 
     case SMB_QUERY_FILE_ALL_INFO:
-      put_long_date(pdata,sbuf.st_ctime); /* create time = inode mod time */
-      put_long_date(pdata+8,sbuf.st_atime); /* access time */
+      put_long_date(pdata,get_create_time(&sbuf,lp_fake_dir_create_times(SNUM(cnum))));
+      put_long_date(pdata+8,sbuf.st_atime);
       put_long_date(pdata+16,sbuf.st_mtime); /* write time */
       put_long_date(pdata+24,sbuf.st_mtime); /* change time */
       SIVAL(pdata,32,mode);
@@ -1347,6 +1473,14 @@ static int call_trans2setfilepathinfo(char *inbuf, char *outbuf, int length,
       tvs.modtime=MAX(interpret_long_date(pdata+16),
                       interpret_long_date(pdata+24));
 
+#if 0 /* Needs more testing... */
+      /* Test from Luke to prevent Win95 from
+         setting incorrect values here.
+       */
+      if (tvs.actime < tvs.modtime)
+        return(ERROR(ERRDOS,ERRnoaccess));
+#endif /* Needs more testing... */
+
       /* attributes */
       mode = IVAL(pdata,32);
       break;
@@ -1383,7 +1517,7 @@ static int call_trans2setfilepathinfo(char *inbuf, char *outbuf, int length,
    */
   if (st.st_mtime != tvs.modtime || st.st_atime != tvs.actime)
   {
-    if(file_utime(cnum,fname, &tvs)!=0)
+    if(file_utime(cnum, fname, &tvs)!=0)
     {
       return(ERROR(ERRDOS,ERRnoaccess));
     }
@@ -1610,6 +1744,19 @@ int reply_trans2(char *inbuf,char *outbuf,int length,int bufsize)
   char *params = NULL, *data = NULL;
   int num_params, num_params_sofar, num_data, num_data_sofar;
 
+  if(global_oplock_break && (tran_call == TRANSACT2_OPEN))
+  {
+    /*
+     * Queue this open message as we are the process of an oplock break.
+     */ 
+
+    DEBUG(2,("%s: reply_trans2: queueing message trans2open due to being in oplock break state.\n",
+           timestring() ));
+
+    push_smb_message( inbuf, length);
+    return -1;
+  }
+
   outsize = set_message(outbuf,0,0,True);
 
   /* All trans2 messages we handle have smb_sucnt == 1 - ensure this
@@ -1652,11 +1799,19 @@ int reply_trans2(char *inbuf,char *outbuf,int length,int bufsize)
 
       while( num_data_sofar < total_data || num_params_sofar < total_params)
 	{
-	  if(!receive_smb(Client,inbuf, SMB_SECONDARY_WAIT) ||
-	     CVAL(inbuf, smb_com) != SMBtranss2)
+          BOOL ret;
+
+          ret = receive_next_smb(Client,oplock_sock,inbuf,bufsize,
+                             SMB_SECONDARY_WAIT);
+
+	  if((ret && (CVAL(inbuf, smb_com) != SMBtranss2)) || !ret)
 	    {
 	      outsize = set_message(outbuf,0,0,True);
-	      DEBUG(2,("Invalid secondary trans2 packet\n"));
+              if(ret)
+                DEBUG(0,("reply_trans2: Invalid secondary trans2 packet\n"));
+              else
+                DEBUG(0,("reply_trans2: %s in getting secondary trans2 response.\n",
+                         (smb_read_error == READ_ERROR) ? "error" : "timeout" ));
 	      free(params);
 	      free(data);
 	      return(ERROR(ERRSRV,ERRerror));
