@@ -2,7 +2,7 @@
    Unix SMB/Netbios implementation.
    Version 1.9.
    uid/user handling
-   Copyright (C) Andrew Tridgell 1992-1997
+   Copyright (C) Andrew Tridgell 1992-1998
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,15 +23,13 @@
 
 extern int DEBUGLEVEL;
 
-extern connection_struct Connections[];
-
 static int initial_uid;
 static int initial_gid;
 
 /* what user is current? */
 struct current_user current_user;
 
-extern pstring OriginalDir;
+pstring OriginalDir;
 
 /****************************************************************************
 initialise the uid routines
@@ -55,6 +53,7 @@ void init_uid(void)
   initial_gid = getegid();
 
   current_user.cnum = -1;
+  current_user.vuid = UID_FIELD_INVALID;
 
   ChDir(OriginalDir);
 }
@@ -176,6 +175,7 @@ BOOL become_guest(void)
     DEBUG(1,("Failed to become guest. Invalid guest account?\n"));
 
   current_user.cnum = -2;
+  current_user.vuid = UID_FIELD_INVALID;
 
   return(ret);
 }
@@ -183,19 +183,19 @@ BOOL become_guest(void)
 /*******************************************************************
 check if a username is OK
 ********************************************************************/
-static BOOL check_user_ok(int cnum,user_struct *vuser,int snum)
+static BOOL check_user_ok(connection_struct *conn, user_struct *vuser,int snum)
 {
   int i;
-  for (i=0;i<Connections[cnum].uid_cache.entries;i++)
-    if (Connections[cnum].uid_cache.list[i] == vuser->uid) return(True);
+  for (i=0;i<conn->uid_cache.entries;i++)
+    if (conn->uid_cache.list[i] == vuser->uid) return(True);
 
   if (!user_ok(vuser->name,snum)) return(False);
 
-  i = Connections[cnum].uid_cache.entries % UID_CACHE_SIZE;
-  Connections[cnum].uid_cache.list[i] = vuser->uid;
+  i = conn->uid_cache.entries % UID_CACHE_SIZE;
+  conn->uid_cache.list[i] = vuser->uid;
 
-  if (Connections[cnum].uid_cache.entries < UID_CACHE_SIZE)
-    Connections[cnum].uid_cache.entries++;
+  if (conn->uid_cache.entries < UID_CACHE_SIZE)
+    conn->uid_cache.entries++;
 
   return(True);
 }
@@ -204,36 +204,55 @@ static BOOL check_user_ok(int cnum,user_struct *vuser,int snum)
 /****************************************************************************
   become the user of a connection number
 ****************************************************************************/
-BOOL become_user(int cnum, uint16 vuid)
+BOOL become_user(connection_struct *conn, int cnum, uint16 vuid)
 {
   user_struct *vuser = get_valid_user_struct(vuid);
   int snum,gid;
   int uid;
 
-  if (current_user.cnum == cnum && vuser != 0 && current_user.id == vuser->uid) {
+  /*
+   * We need a separate check in security=share mode due to vuid
+   * always being UID_FIELD_INVALID. If we don't do this then
+   * in share mode security we are *always* changing uid's between
+   * SMB's - this hurts performance - Badly.
+   */
+
+  if((lp_security() == SEC_SHARE) && (current_user.cnum == cnum) &&
+     (current_user.uid == conn->uid)) {
+    DEBUG(4,("Skipping become_user - already user\n"));
+    return(True);
+  } else if ((current_user.cnum == cnum) && (vuser != 0) && (current_user.vuid == vuid) && 
+      (current_user.uid == vuser->uid)) {
     DEBUG(4,("Skipping become_user - already user\n"));
     return(True);
   }
 
   unbecome_user();
 
-  if (!OPEN_CNUM(cnum)) {
+  if (!(VALID_CNUM(cnum) && conn->open)) {
     DEBUG(2,("Connection %d not open\n",cnum));
     return(False);
   }
 
-  snum = Connections[cnum].service;
+  snum = conn->service;
 
-  if (Connections[cnum].force_user || 
+  if((vuser != NULL) && !check_user_ok(conn, vuser, snum))
+    return False;
+
+  if (conn->force_user || 
       lp_security() == SEC_SHARE ||
-      !(vuser) || (vuser->guest) ||
-      !check_user_ok(cnum,vuser,snum)) {
-    uid = Connections[cnum].uid;
-    gid = Connections[cnum].gid;
-    current_user.groups = Connections[cnum].groups;
-    current_user.igroups = Connections[cnum].igroups;
-    current_user.ngroups = Connections[cnum].ngroups;
-  } else {
+      !(vuser) || (vuser->guest)
+     )
+  {
+    uid = conn->uid;
+    gid = conn->gid;
+    current_user.groups = conn->groups;
+    current_user.igroups = conn->igroups;
+    current_user.ngroups = conn->ngroups;
+    current_user.attrs   = conn->attrs;
+  }
+  else
+  {
     if (!vuser) {
       DEBUG(2,("Invalid vuid used %d\n",vuid));
       return(False);
@@ -242,10 +261,11 @@ BOOL become_user(int cnum, uint16 vuid)
     if(!*lp_force_group(snum))
       gid = vuser->gid;
     else
-      gid = Connections[cnum].gid;
-    current_user.groups = vuser->user_groups;
-    current_user.igroups = vuser->user_igroups;
-    current_user.ngroups = vuser->user_ngroups;
+      gid = conn->gid;
+    current_user.ngroups = vuser->n_groups;
+    current_user.groups  = vuser->groups;
+    current_user.igroups = vuser->igroups;
+    current_user.attrs   = vuser->attrs;
   }
 
   if (initial_uid == 0)
@@ -253,22 +273,20 @@ BOOL become_user(int cnum, uint16 vuid)
       if (!become_gid(gid)) return(False);
 
 #ifndef NO_SETGROUPS      
-      if (!IS_IPC(cnum)) {
+      if (!(VALID_CNUM(cnum) && conn->ipc)) {
 	/* groups stuff added by ih/wreu */
 	if (current_user.ngroups > 0)
 	  if (setgroups(current_user.ngroups,current_user.groups)<0)
 	    DEBUG(0,("setgroups call failed!\n"));
-      } else {
-	    current_user.ngroups = 0;  
       }
 #endif
 
-      if (!Connections[cnum].admin_user && !become_uid(uid))
+      if (!conn->admin_user && !become_uid(uid))
 	return(False);
     }
 
   current_user.cnum = cnum;
-  current_user.id = uid;
+  current_user.vuid = vuid;
   
   DEBUG(5,("become_user uid=(%d,%d) gid=(%d,%d)\n",
 	   getuid(),geteuid(),getgid(),getegid()));
@@ -320,8 +338,7 @@ BOOL unbecome_user(void )
 
   current_user.uid = initial_uid;
   current_user.gid = initial_gid;
-  current_user.ngroups = 0;
-
+  
   if (ChDir(OriginalDir) != 0)
     DEBUG(0,("%s chdir(%s) failed in unbecome_user\n",
 	     timestring(),OriginalDir));
@@ -330,6 +347,7 @@ BOOL unbecome_user(void )
 	getuid(),geteuid(),getgid(),getegid()));
 
   current_user.cnum = -1;
+  current_user.vuid = UID_FIELD_INVALID;
 
   return(True);
 }
@@ -413,7 +431,7 @@ int smbrun(char *cmd,char *outfile,BOOL shared)
       return(1);
     }
 
-  sprintf(syscmd,"%s %d %d \"(%s 2>&1) > %s\"",
+  slprintf(syscmd,sizeof(syscmd)-1,"%s %d %d \"(%s 2>&1) > %s\"",
 	  path,uid,gid,cmd,
 	  outfile?outfile:"/dev/null");
 
@@ -480,8 +498,6 @@ int smbrun(char *cmd,char *outfile,BOOL shared)
   return 1;
 }
 
-
-
 static struct current_user current_user_saved;
 static int become_root_depth;
 static pstring become_root_dir;
@@ -494,7 +510,7 @@ after the operation
 
 Set save_dir if you also need to save/restore the CWD 
 ****************************************************************************/
-void become_root(int save_dir) 
+void become_root(BOOL save_dir) 
 {
 	if (become_root_depth) {
 		DEBUG(0,("ERROR: become root depth is non zero\n"));
@@ -505,8 +521,8 @@ void become_root(int save_dir)
 	current_user_saved = current_user;
 	become_root_depth = 1;
 
-	become_gid(0);
 	become_uid(0);
+	become_gid(0);
 }
 
 /****************************************************************************
@@ -514,7 +530,7 @@ When the privilaged operation is over call this
 
 Set save_dir if you also need to save/restore the CWD 
 ****************************************************************************/
-void unbecome_root(int restore_dir)
+void unbecome_root(BOOL restore_dir)
 {
 	if (become_root_depth != 1) {
 		DEBUG(0,("ERROR: unbecome root depth is %d\n",
@@ -555,6 +571,3 @@ void unbecome_root(int restore_dir)
 
 	become_root_depth = 0;
 }
-
-
-
